#include <stdio.h>
#include "Encoder.hpp"
#include <ni_rsrc_api.h>

//HEIF includes

#include "isomediafile.hpp"
#include <cstdlib>
#include <iostream>

#include "json/json.h"
#include <fstream>

#define MB 1000000
using namespace std;

#define ODD2EVEN(X) (X&1)?(X+1):(X)

// 1.8 GB file cache
#define MAX_CACHE_FILE_SIZE 0x70000000

Encoder::Encoder():ICodec(ENCODER)
{
  m_frameSize=0;
  m_loadpixel = 0;
  m_height_aligned=0;
  memset(m_height_aligned_array,0,sizeof(int)*4);
  ni_packet_buffer_alloc(&(m_out_packet.data.packet), NI_MAX_TX_SZ);
  if (!m_out_packet.data.packet.p_data)
  {
    printf("Error allocating buffer!!\n");
  }
}
Encoder::~Encoder()
{
  if(nullptr!=m_data)
  {
    delete [] m_data;
    m_data_len =0;
  }
}

void fillFrame(int width,int height,int *dst_stride,int * height_aligned, unsigned char** p_src,ni_frame_t* p_in_frame)
{
  int plane_height[4]={0};
  int src_stride[4]={0};
  src_stride[0] = width;
  src_stride[1] = src_stride[2] = src_stride[0] / 2;
  plane_height[0] = height;
  plane_height[1] = plane_height[2] = height / 2;
  plane_height[3] = 0;
  for (auto i = 0; i < 3; i++)
  {
    int height = plane_height[i];
    int byte = src_stride[i] < dst_stride[i] ? src_stride[i] : dst_stride[i];
    if ((i>0) &&(width&1)){
      src_stride[i] = ODD2EVEN(src_stride[i])+1;
    }
    unsigned char* src = p_src[i];
    unsigned char* dst=(unsigned char*)p_in_frame->p_data[i];
    for (; height > 0; height--)
    {
      memcpy(dst, src,byte);
      dst += dst_stride[i];
      src += src_stride[i];
    }
    // height padding if needed
    int padding_height = height_aligned[i] - plane_height[i];
    if (padding_height > 0)
    {
      src -= src_stride[i];
      for (; padding_height > 0; padding_height--)
      {
        memcpy(dst, src,byte);
        dst += dst_stride[i];
      }
    }
  }
}

int Encoder::resetSession()
{
  memset(&(m_out_packet.data.packet),0,sizeof(m_out_packet.data.packet));
  ni_packet_buffer_alloc(&(m_out_packet.data.packet), NI_MAX_TX_SZ);
  if (!m_out_packet.data.packet.p_data)
  {
    printf("Error allocating buffer!!\n");
    return NI_RETCODE_FAILURE;
  }
  m_firstFrame=true;
  m_session_ctx->ready_to_close=0;
  m_eos_sent = m_eos_received = m_resendFlag =0;
  m_sentCnt = m_receivedCnt =0;
  m_header_len=0;
  m_firstRecv=true;
  m_frameSize=0;
  m_loadpixel = 0;
  m_height_aligned=0;
  m_width_aligned=0;
  memset(m_height_aligned_array,0,sizeof(int)*4);

  if(0 != closeSession())
  {
    printf("close session failed %s\n",__func__);
    return NI_RETCODE_FAILURE;
  }
  return NI_RETCODE_SUCCESS;
}

int Encoder::init(ni_codec_format_t codec,
                  const char * destFile,
                  int width,
                  int height,
                  int fps,
                  int bitrate)
{
  int ret = NI_RETCODE_SUCCESS;
  ni_packet_buffer_alloc(&(m_out_packet.data.packet), NI_MAX_TX_SZ);
  if (!m_out_packet.data.packet.p_data)
  {
    printf("Error allocating buffer!!\n");
    return NI_RETCODE_FAILURE;
  }
  ret =ICodec::init(codec,destFile,width,height,fps,bitrate);

  if(ret >= 0)
  {
    return init_encode_default_param(width,height,fps,bitrate);
  }
  return ret ;
}

int Encoder::init_encode_default_param(int width,int height,int fps,int bitrate)
{

  int ret=NI_RETCODE_SUCCESS;
  m_width= width;
  m_height = height;
  m_fps=fps;
  m_bitrate=bitrate;
  m_width_aligned = ((width + 31) / 32) * 32;
  if(  NI_RETCODE_SUCCESS!=(ret=ni_encoder_init_default_params(&m_api_param, m_fps, 1, m_bitrate, m_width, m_height)))
  {
    printf("ni_encoder_init_default_params failed ret = %d\n",ret);
    return NI_RETCODE_FAILURE;
  }

  if (m_width_aligned < NI_MIN_WIDTH)
  {
    m_api_param.hevc_enc_params.conf_win_right += NI_MIN_WIDTH  - m_width;
    m_width_aligned = NI_MIN_WIDTH;
  }
  else if(m_width_aligned >width)
  {
    m_api_param.hevc_enc_params.conf_win_right += m_width_aligned - m_width;
  }

  m_api_param.source_width = m_width_aligned;
  m_height_aligned = ((m_height + 7) / 8) * 8;
  if (m_session_ctx->codec_format == NI_CODEC_FORMAT_H264)
  {
    m_height_aligned = ((m_height + 15) / 16) * 16;
  }
  if (m_height_aligned < NI_MIN_HEIGHT)
  {
    m_api_param.hevc_enc_params.conf_win_bottom += NI_MIN_HEIGHT - m_height;
    m_api_param.source_height = NI_MIN_HEIGHT;
    m_height_aligned = NI_MIN_HEIGHT;
  }
  else if (m_height_aligned > m_height)
  {
    m_api_param.hevc_enc_params.conf_win_bottom += m_height_aligned - m_height;
    if (m_session_ctx->codec_format != NI_CODEC_FORMAT_H264)
    {
      m_api_param.source_height = m_height_aligned;
    }
  }
  m_api_param.hevc_enc_params.conf_win_right = ODD2EVEN(m_api_param.hevc_enc_params.conf_win_right);
  m_api_param.hevc_enc_params.conf_win_bottom = ODD2EVEN(m_api_param.hevc_enc_params.conf_win_bottom);
  m_session_ctx->p_session_config =&m_api_param;
  m_height_aligned_array[0] = m_height_aligned;
  m_height_aligned_array[1] = m_height_aligned_array[2] = m_height_aligned_array[0] / 2;
  m_dst_stride[0] = m_width_aligned*m_session_ctx->bit_depth_factor;
  m_dst_stride[1]=m_dst_stride[2]=m_width_aligned/2;
  m_frameSize = m_width * m_height * 3 / 2;
  m_header_len=0;
  m_firstRecv=true;

  //init vui
  m_api_param.ui8VuiRbsp[0] = 128;
  m_api_param.ui8VuiRbsp[1] = 182;
  m_api_param.ui8VuiRbsp[2] = 4;
  m_api_param.pos_num_units_in_tick = 22;
  m_api_param.pos_time_scale = 54;
  m_api_param.ui32VuiDataSizeBits = 89;
  m_api_param.ui32VuiDataSizeBytes = 12;
  //10000000 10110110 000001.00 00000000 00000000 00000000 000000.00 00000000 00000000 00000000 000000.00 0.0000000 
  //128      182      4         0        0        0        0         0        0        0        0         0

  return ret ;
}


void Encoder::setParam(const char * key,const char * value)
{
  ni_retcode_t ret;

  ret=ni_encoder_params_set_value(&m_api_param, key, value);
  if ( ret == NI_RETCODE_PARAM_INVALID_NAME)
  {
    printf("Unknown option: %s.\n", key);
  }
  if ( ret == NI_RETCODE_PARAM_INVALID_VALUE)
  {
    printf("Unknown option: %s.\n", value);
  }
}

int Encoder::allocateBuffer(unsigned char ** buffer,unsigned int len,unsigned int extraSize)
{
  int i;
  int ret = NI_RETCODE_SUCCESS;

  ni_frame_t * p_in_frame =  &(m_in_frame.data.frame);
  int nb_planes = 3;

  p_in_frame->start_of_stream = 0;
  p_in_frame->extra_data_len = NI_APP_ENC_FRAME_META_DATA_SIZE + extraSize;
  p_in_frame->reconf_len = 0;
  if(m_firstFrame)
  {
    m_firstFrame=false;
    p_in_frame->start_of_stream = 1;
  }
  p_in_frame->end_of_stream = 0;
  p_in_frame->force_key_frame = 1; 
  if (m_dst_stride[0] < NI_MIN_WIDTH)
  {
    m_dst_stride[0] = NI_MIN_WIDTH;
  }
  m_dst_stride[1] = m_dst_stride[2] = m_dst_stride[0] / 2;
  if (len == 0) {
    p_in_frame->end_of_stream = 1;
    p_in_frame->data_len[0]=p_in_frame->data_len[1]=p_in_frame->data_len[2]=0;
    //cout<<"-------End of stream, "<< m_dest.getFilePath() <<endl;
  }
  else
  {
    p_in_frame->video_width = m_width;
    p_in_frame->video_height = m_height;
    p_in_frame->ni_pict_type = PIC_TYPE_IDR;
    p_in_frame->force_key_frame = 1;
    ni_frame_buffer_alloc_v3(p_in_frame, ODD2EVEN(m_width), ODD2EVEN(m_height), m_dst_stride, (m_session_ctx->codec_format == NI_CODEC_FORMAT_H264),p_in_frame->extra_data_len);
    if(nullptr!=p_in_frame->p_data[0])
    {
      fillFrame(m_width,m_height,m_dst_stride,m_height_aligned_array,buffer,p_in_frame);
    }
    else
    {
      printf("ni_frame_buffer_alloc_v3 failed\n");
      ret = NI_RETCODE_FAILURE;
    }
  }
  return ret;
}

int Encoder::sendData()
{
  m_resendFlag = false;
  int ret = NI_RETCODE_SUCCESS;
  if (m_eos_sent!=0)
  {
    return 0;
  }

  int flush = (m_in_frame.data.frame).end_of_stream;
  int sentFlag= ni_device_session_write(m_session_ctx, &m_in_frame, NI_DEVICE_TYPE_ENCODER);

  if (NI_RETCODE_ERROR_RESOURCE_UNAVAILABLE == sentFlag || NI_RETCODE_FAILURE == sentFlag)
  {
    m_resendFlag = true;
    sentFlag = 0;
  }
  if (m_session_ctx->ready_to_close||flush)
  {
    m_eos_sent = 1;
    ret = ni_device_session_flush(m_session_ctx, NI_DEVICE_TYPE_ENCODER);
    if (ret < 0)
    {
      printf("Failed to flush encoder (status = %d)\n", ret);
      return ret;
    }

    //printf("encoder_send_data: eos sent\n");
  }
  if(sentFlag < 0)
  {
    printf("Sent error %d \n",sentFlag);
  }
  else if(sentFlag >0)
  {
    m_sentCnt ++;
  }

  return sentFlag;
}
int Encoder::receiveData()
{
  int rc = NI_RETCODE_SUCCESS;
  if(m_eos_received==1)
  {
    return rc;
  }
  int packet_size = NI_MAX_TX_SZ;
  int rx_size = 0;
  ni_packet_t * p_out_pkt =  &(m_out_packet.data.packet);
  int meta_size = NI_FW_ENC_BITSTREAM_META_DATA_SIZE;

  p_out_pkt->data_len =0;
  rc = ni_device_session_read(m_session_ctx, &m_out_packet, NI_DEVICE_TYPE_ENCODER);

  rx_size = p_out_pkt->data_len;

  if(rx_size > meta_size)
  {
    unsigned int writeLen = p_out_pkt->data_len - meta_size;

    if(m_header_len>0)
    {
      if(writeLen==m_header_len)
      {
        printf("#######repeated header\n");
      }
      else
      {
        if(m_heif_list.size()>0)
        {
          if(m_repeated_header==0)
          {
            if((writeLen+m_header_len)> m_data_len)///re-allocate buffer
            {
              uint8_t * tmp = m_data;
              m_data_len = writeLen+m_header_len;
              m_data = new uint8_t[m_data_len];
              memcpy(m_data,tmp,m_header_len);
              delete [] tmp;
            }
            memcpy(m_data+m_header_len,(unsigned char*)p_out_pkt->p_data + meta_size,writeLen);
            m_heif_writer.app_processConfiguration(m_heif_list.front(),m_data,writeLen+m_header_len);
#if 0
            FILE * dump = fopen("dump.h265", "wb");
            fwrite(m_data, writeLen+m_header_len, 1, dump);
            fclose(dump);
#endif
          }
          else
          {
            if(!m_firstRecv)
            {
              m_heif_writer.app_processConfiguration(m_heif_list.front(),(unsigned char*)p_out_pkt->p_data + meta_size,writeLen);
            }
            else
            {
              // first frame come without headers we concat the headers to the packet
              memcpy(m_data+m_header_len,(unsigned char*)p_out_pkt->p_data + meta_size,writeLen);
              m_heif_writer.app_processConfiguration(m_heif_list.front(),m_data,writeLen+m_header_len);
              m_firstRecv = false;
            }
          }
          m_heif_list.pop_front();
        }
        else
        {
          printf("########HEIF file name null\n");
        }
        m_receivedCnt ++;
      }
    }
    else
    {
      // at first execution the encoder returns only the headers, we copy them into m_data
      m_header_len = writeLen;
      memcpy(m_data,(unsigned char*)p_out_pkt->p_data + meta_size,writeLen);
#ifdef DEBUG
      printf("#########header data len %d [%s]\n",writeLen,m_dest.getFilePath());
#endif
    }
  }
  else if (rx_size != 0)
  {
    printf("Error: received %d bytes, <= metadata size %d!\n", rx_size, meta_size);
  }
  m_eos_received = p_out_pkt->end_of_stream;
  if (m_eos_received)
  {
    //cout<<"Encoding Receiving done"<<endl;
    ni_packet_buffer_free(p_out_pkt);
  }
  return rc;
}
