
#include "FileSource.hpp"
using namespace std;
FileSource::FileSource( int mode)
{
  m_filehandle = -1;
  m_pBuffer=NULL;
  m_NalHeader_Len =0;
  memset(m_filePath,0,512);
  m_mode =mode;
  m_length=0;

}


void FileSource::resetReadPos()
{
  m_readPos = m_pBuffer;
}

const char* FileSource::getFilePath()const
{
  return m_filePath;
}

int FileSource::open(char const* file_path)
{
   strcpy(m_filePath,file_path);
   mode_t mode = S_IRWXU | S_IRWXG | S_IROTH;
   if (m_mode == O_RDONLY)
   {
     m_filehandle=::open(m_filePath,m_mode,mode);
     if(m_filehandle<=0)
     {
       printf("open %s failed ret = %d\n",m_filePath,m_filehandle);
       return -1;
     }
     struct stat st;
    if(-1==fstat(m_filehandle,&st))
    {
       printf("get file %s status failed\n",m_filePath);
       return -1;
    }
     m_length = st.st_size;
     m_pBuffer=(unsigned char*)mmap(NULL,m_length,PROT_READ,MAP_SHARED,m_filehandle,0);
     if(m_pBuffer==nullptr)
     {
       printf("mapping file %s failed\n",m_filePath);
       return -1;
     }
     m_readPos = m_pBuffer;
     //printf("Open source %s, size = %llu\n",m_filePath,m_length);
   }
   else
   {
     m_filehandle=::open(m_filePath,m_mode|O_CREAT|O_TRUNC ,mode);
     if(m_filehandle<=0)
     {
       printf("open %s failed ret = %d\n",m_filePath,m_filehandle);
       return -1;
     }
     //printf("open destination file %s,\n",m_filePath);
   }
   return 0;
}

void FileSource::close()
{
  if(m_pBuffer != nullptr)
  {
    munmap(m_pBuffer,m_length);
    m_pBuffer = nullptr;
  }
  if(-1 != m_filehandle)
  {
    if(m_mode&&O_WRONLY)
    {
      getFileSize();
      //printf("Close file[%s] %llu bytes\n",m_filePath,m_length);
    }
    ::close(m_filehandle);
    m_filehandle = -1;
  }
  m_pBuffer=NULL;
  m_NalHeader_Len =0;
  m_length =0;
  memset(m_filePath,0,512);
}

FileSource::~FileSource()
{
  close();

}
unsigned char * FileSource::read(unsigned int &readLen )
{
  unsigned char *ptr = NULL;
  if(m_mode==O_RDONLY)
  {
    if ((m_pBuffer==nullptr)||(m_readPos -m_pBuffer)<m_length)
    {
      unsigned long long leftLen = m_length - ((unsigned long long)(m_readPos-m_pBuffer)) ;
      if (leftLen< readLen)
      {
        readLen = leftLen;
      }
      ptr = m_readPos;
      m_readPos+= readLen;
    }
    else
      readLen = 0;
  }
  else
  {
    readLen=0;
  }
  return ptr;
}

unsigned int FileSource::write(unsigned char * pbuffer,unsigned int writeLen)
{
    int ret =0;
  if(m_mode==O_RDONLY || pbuffer==nullptr)
  {
    return ret;
  }
    ret= ::write(m_filehandle,(void*)pbuffer,writeLen);
    return ret;
}

unsigned int FileSource::getFileSize()
{
  if(-1 != m_filehandle)
  {
    fsync(m_filehandle);
    struct stat st;
    fstat(m_filehandle,&st);
    m_length = st.st_size;
  }
  return m_length;
}
