/*
 * ICodec.cpp
 *
 *  Created on: 2019-10-04
 *      Author: root
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdint.h>
#include <iostream>
#include <chrono>
#include <thread>
#include "ICodec.hpp"

using namespace std;
map<string,int> ICodec::m_deviceHandles;
std::map<string,int> ICodec::m_deviceIOSizes;

int ICodec::OpenDevice(const char * device_name,const char* device_blk_name)
{
  ni_device_handle_t dev_guid, dev_name;
  unsigned int maxIOSize = 0;
  if(NI_INVALID_DEVICE_HANDLE==(dev_guid= ni_device_open(device_name,&maxIOSize)))
  {
    printf("ni_device_open %s failed\n",device_name);
    return NI_RETCODE_FAILURE;
  }
  printf("ICodec::OpenDevice %s successfully\n",device_name);
  m_deviceHandles.insert(make_pair(device_name,dev_guid));
  m_deviceIOSizes.insert(make_pair(device_name,maxIOSize));
  if(NI_INVALID_DEVICE_HANDLE==(dev_name = ni_device_open(device_blk_name,&maxIOSize)))
  {
    printf("ni_device_open %s failed\n",device_blk_name);
    return NI_RETCODE_FAILURE;
  }
  printf("ICodec::OpenDevice %s successfully\n",device_blk_name);
  m_deviceHandles.insert(make_pair(device_blk_name,dev_name));
 // m_deviceIOSizes.insert(make_pair(device_blk_name,maxIOSize));
  return NI_RETCODE_SUCCESS;
}

int ICodec::CloseDevice(const char * device_name,const char* device_blk_name)
{
  int ret = NI_RETCODE_SUCCESS;
  ni_device_close(m_deviceHandles[device_name]);
  ni_device_close(m_deviceHandles[device_blk_name]);
  m_deviceHandles.erase(device_name);
  m_deviceHandles.erase(device_blk_name);
  m_deviceIOSizes.erase(device_name);
  //m_deviceIOSizes.erase(device_blk_name);
  return ret;
}


ICodec::ICodec (CODEC_TYPE codec) :
        m_dest(O_WRONLY)
{
  m_codec=codec;
  m_device_handle = 0;
  m_eos_sent=0;
  m_eos_received=0;
  m_width=0;
  m_height=0;
  m_fps =0;
  m_bitrate=0;
  memset(&m_api_param,0,sizeof(ni_encoder_params_t));
  m_session_ctx = nullptr;
  memset(&m_in_frame,0,sizeof(ni_session_data_io_t));
  memset(&m_out_packet,0,sizeof(ni_session_data_io_t));
  m_receivedCnt = 0;
  m_sentCnt = 0;
  m_recvActive = false;
}

ICodec::~ICodec() {
  // TODO Auto-generated destructor stub
  m_dest.close();
}

int ICodec::init(  ni_codec_format_t codec,
  const char * destFile,
  int width,
  int height,
  int fps,
  int bitrate
)
{
  m_width= width;
  m_height = height;
  m_encoderFormat =codec;
  m_fps=fps;
  m_bitrate=bitrate;
  m_dest.close();
  m_dest.open(destFile);
  m_session_ctx = ni_device_session_context_alloc_init();
  if(nullptr==m_session_ctx)
  {
    printf("ni_device_session_context_alloc_init failed\n");
    return NI_RETCODE_FAILURE;
  }
  m_session_ctx->codec_format = m_encoderFormat;
  m_session_ctx->src_bit_depth = 8;
  m_session_ctx->src_endian = NI_FRAME_LITTLE_ENDIAN;
  m_session_ctx->session_id = -1;
  m_session_ctx->bit_depth_factor =1;
  m_firstFrame = true;
  m_resendFlag=false;
  printf("%s Init: %dx%d, bitrate=%d, fps=%d, dest=%s\n",m_codec==ENCODER?"Encoder":"Decoder",
        width,
        height,
        bitrate,
        fps,
        destFile);
  return NI_RETCODE_SUCCESS;
}

int ICodec::assignDevice(int hw_id)
{
  int ret = NI_RETCODE_SUCCESS;

  if (nullptr!= m_session_ctx)
  {
    m_session_ctx->hw_id = hw_id;
  }
  else
  {
    printf("unsucessful device assignment for enc %d\n", hw_id);
    ret = NI_RETCODE_FAILURE;
  }
  return ret;
}

int ICodec::removeDevice()
{
  int ret = NI_RETCODE_SUCCESS;

  ret = ni_frame_buffer_free( &(m_in_frame.data.frame) );
  ret = ni_packet_buffer_free( &(m_out_packet.data.packet) );

  return ret;
}

int ICodec::openSession()
{
  int ret = NI_RETCODE_SUCCESS;
  ni_device_type_t dev_type = (m_codec == ENCODER)? NI_DEVICE_TYPE_ENCODER:NI_DEVICE_TYPE_DECODER ;

  if(NI_RETCODE_SUCCESS!=(ret=ni_device_session_open(m_session_ctx, dev_type)))
  {
    printf("open %s failed\n",(m_codec == ENCODER)?"Encoder":"Decoder");
    return ret;
  }
#ifdef DEBUG
  printf("open %s session ID: %d\n",(m_codec == ENCODER)?"Encoder":"Decoder",m_session_ctx->session_id);
#endif
  return ret;
}

int ICodec::closeSession()
{
  int ret = NI_RETCODE_SUCCESS;
  ni_device_type_t dev_type = m_codec == DECODER? NI_DEVICE_TYPE_DECODER :NI_DEVICE_TYPE_ENCODER;
  int id = m_session_ctx->session_id;
  if( NI_RETCODE_SUCCESS==(ret=ni_device_session_close(m_session_ctx, m_eos_received, dev_type)))
  {
    m_session_ctx->session_id =-1;
  }
#ifdef DEBUG
  printf("%s to close Session ID: %d\n",ret==NI_RETCODE_SUCCESS?"succeeded":"failed", id);
#endif
  return ret;
}

int ICodec::startRecv(bool bindCore,int core)
{
  m_recvActive = true;
  int rc = NI_RETCODE_SUCCESS;
  rc = pthread_create(&m_recvThread,NULL,ICodec::receiveProc,this);
  if(rc!=0)
  {
    printf("create receive data thread failed: %d\n",rc);
    m_recvActive = false;
    return NI_RETCODE_FAILURE;
  }
  else if(bindCore)
  {
    // printf("create receive data thread: %llu \n", m_recvThread);
    cpu_set_t cpuset;
    CPU_ZERO(&cpuset);
    CPU_SET(core, &cpuset);
    int rc = pthread_setaffinity_np(m_recvThread,
                                    sizeof(cpu_set_t), &cpuset);
    if (rc != 0)
    {
      std::cerr << "Error calling pthread_setaffinity_np: " << rc << "\n";
    }
  }
  return rc;
}

int ICodec::stopRecv(bool wait)
{
  int rc = NI_RETCODE_SUCCESS;
  if(!m_recvActive)
  {
    return rc;
  }
  rc = ni_device_session_flush(m_session_ctx, m_codec==ENCODER?NI_DEVICE_TYPE_ENCODER:NI_DEVICE_TYPE_DECODER);
  if (rc < 0)
  {
    printf("Failed to flush %s (status = %d)\n", m_codec==ENCODER?"encoder":"decoder", rc);
    return rc;
  }

  void * status;
  if(!wait)
  {
    m_recvActive = false;
    pthread_cancel(m_recvThread);

  }
  rc = pthread_join(m_recvThread,&status);
  if(rc!=0)
  {
    printf("wait receive data thread failed: %d\n",rc);
  }
  m_recvActive =false;
  //printf("stop thread :%llu\n",m_recvThread);
  return 0;
}

void * ICodec::receiveProc(void *pVoid)
{
  ICodec * pCodec = (ICodec*)pVoid;

  //printf("receiveProc started on CPU %d\n",sched_getcpu());
  while(pCodec->m_recvActive&&pCodec->m_eos_received == 0)
  {
    int receive_flag =pCodec->receiveData();
    if (receive_flag<0)
    {
      printf("receive data failed receive flag: %d\n",receive_flag);
      break;
    }
  }
  printf("receiveProc[%s] stopped on CPU %d, total %d frames\n",pCodec->m_dest.getFilePath() ,sched_getcpu(),pCodec->m_receivedCnt);
  pthread_exit(nullptr);
}
