/*
 * ImageToYuv.cpp
 *
 *  Created on: May 25, 2020
 *      Author: ray
 */
#include <chrono>
#include <unistd.h>
#include "ImageToYuv.hpp"
#include "YuvToHeic.hpp"

ImageToYuv::ImageToYuv() {
	// TODO Auto-generated constructor stub

}

ImageToYuv::~ImageToYuv() {
	// TODO Auto-generated destructor stub
}


int ImageToYuv::Start(int taskID,void (*callback)(int), bool bindCore)
{
  m_bActive = true;
  m_endCb = callback;
  int rc = 0;
  //spawn a thread for decoding frame to yuv 
  rc = pthread_create(&m_thread,NULL,&ImageToYuv::transProc,this);
  if(rc!=0)
  {
    printf("create receive data thread failed: %d\n",rc);
    m_bActive = false;
    return -1;
  }
  else
  {
    //printf("create receive data thread : %llu\n",m_thread);
  }
  if(bindCore)
  {
    cpu_set_t cpuset;
    CPU_ZERO(&cpuset);
    CPU_SET(taskID, &cpuset);
    rc = pthread_setaffinity_np(m_thread,sizeof(cpu_set_t), &cpuset);
    if (rc != 0) {
      printf("Error calling pthread_setaffinity_np: %d\n" ,rc );
    }
    else
    {
      //printf("Binding thread %llu to CPU %d\n",m_thread,taskID);
    }
  }
  m_taskID= taskID;
 
  return rc;
}

int ImageToYuv::Stop(bool wait)
{
  int ret = 0;
  if(!m_bActive)
  {
    return 0;
  }
  if(!wait)
  {
    m_bActive = false;
    pthread_cancel(m_thread);
  }
  void * status;
  //printf("wait receive data thread to finish\n");
  ret = pthread_join(m_thread,&status);
  if(ret!=0)
  {
    printf("wait receive data thread failed: %d\n",ret);
    ret = -1;
  }
  //printf("stop thread :%llu\n",m_thread);
  m_bActive = false;
  avformat_free_context(m_avfmt_ctx);
  return ret;
}

int ImageToYuv::Setup(Codec_Param &init_param,SOURCETYPE source,YuvToHeic *heicProc)
{
  av_register_all();
  m_avfmt_ctx = avformat_alloc_context();
  if(m_avfmt_ctx == nullptr)
  {
    printf("error alloc context\n");
    return -1;
  }
  switch (source)
  {
    case JPEG_FILE:
      m_in_fmt = av_find_input_format("mjpeg");
      break;
    case BMP_FILE:
      m_in_fmt = av_find_input_format("bmp");
      break;
    case PNG_FILE:
      m_in_fmt = av_find_input_format("png");
      break;
    default:
      return -1;
  }
  m_in_type = source;
  m_bScaler = init_param.scaler;
  m_height = init_param.height;
  m_width = init_param.width;
  strcpy(m_output_path,init_param.dest_path);
  m_pHeicProc = heicProc;
  return 0;
}

int ImageToYuv::AddTask(const char*pSrc)
{
  m_task.push_back(pSrc);
  return 0;
}

void ImageToYuv::SendEOF()
{
  m_task.push_back("");
}

int ImageToYuv::imageToYuv(const char * image, Encode_Task & task)
{
  SwsContext *swsCtx = NULL;
  AVCodecContext *dec_ctx= NULL;
  int ret = 0;
  AVPacket packet = { 0 };
  AVFrame *frame = NULL;
  AVStream *st;
  enum AVMediaType type;
  unsigned int stream_index;
  unsigned int i;
  int got_frame;

  ret = avformat_open_input(&m_avfmt_ctx, image, m_in_fmt, NULL);
  if(ret)
  {
    printf("error opening file\n");
    return -1;
  }
  ret = avformat_find_stream_info(m_avfmt_ctx, NULL);
  if(m_avfmt_ctx->streams[0]->codecpar == NULL)
  {
    printf("codec par stream 0 is null \n");
  }
  if(ret)
  {
    printf("error finding stream info\n");
    return -1;
  }
  int video_stream_idx = 0;
  dec_ctx = m_avfmt_ctx->streams[video_stream_idx]->codec;
  AVCodecParameters* cdc_params = m_avfmt_ctx->streams[video_stream_idx]->codecpar;
  AVCodec* codec = avcodec_find_decoder(cdc_params->codec_id);
  ret = avcodec_open2(dec_ctx, codec, NULL);
  if (ret < 0)
  {
    printf("Error opening decoder!\n");
    return -1;
  }
  #if 0//DEBUG
    av_dump_format(m_avfmt_ctx, 0, image, 0);

    printf("dec context allocated \n");
  #endif
  av_init_packet(&packet);
  ret = av_read_frame(m_avfmt_ctx, &packet);
  if (ret < 0)
  {
    printf("Error reading packet!\n");
    return -1;
  }
  frame = av_frame_alloc();
  if (!frame)
  {
    printf("error allocating frame\n");
    return -1;
  }
  ret = avcodec_decode_video2(dec_ctx, frame,&got_frame, &packet);
  if (ret < 0)
  {
    printf("Error decoding packet!\n");
    return -1;
  }
  if(!m_bScaler)
  {
    task.width = dec_ctx->width;
    task.height = dec_ctx->height;
  }
  else 
  {
    task.width = m_width;
    task.height = m_height;
  }
  if(task.width > NI_MAX_WIDTH || task.height > NI_MAX_HEIGHT)
  {
    av_freep(&frame);
    avformat_close_input(&m_avfmt_ctx);
    printf("resolution [%dx%d] exceeds maximum\n",task.width,task.height);
    return -1;
  }

  // AVPixelFormat dst_pixel_format = AV_PIX_FMT_YUVJ420P;
  AVPixelFormat dst_pixel_format = AV_PIX_FMT_YUV420P;
  int dst_linesize[4];
  ret = av_image_alloc(task.yuv_data, dst_linesize, task.width, task.height , dst_pixel_format, 1);
  if (ret < 0)
  {
    printf("Error allocating dest image!\n");
    return -1;
  }

  AVPixelFormat src_pixel_format;
  int dummy[4];
  int srcRange, dstRange;
  int brightness, contrast, saturation;
  const int* coefs;
  
  switch (dec_ctx->pix_fmt)
  {
    case AV_PIX_FMT_YUVJ420P:
      src_pixel_format = AV_PIX_FMT_YUV420P;
      srcRange = 1;
      break;
    case AV_PIX_FMT_YUVJ422P:
      src_pixel_format = AV_PIX_FMT_YUV422P;
      srcRange = 1;
      break;
    case AV_PIX_FMT_YUVJ444P:
      src_pixel_format = AV_PIX_FMT_YUV444P;
      srcRange = 1;
      break;
    case AV_PIX_FMT_YUVJ440P:
      src_pixel_format = AV_PIX_FMT_YUV440P;
      srcRange = 1;
      break;
    default:
      src_pixel_format = dec_ctx->pix_fmt;
      srcRange = 0;
  }
  // initialize SWS context for software scaling
  // convert any input format to YUV420 using libav swscaler
  // swsCtx = sws_getCachedContext(swsCtx, dec_ctx->width, dec_ctx->height, dec_ctx->pix_fmt,
  swsCtx = sws_getCachedContext(swsCtx, dec_ctx->width, dec_ctx->height, src_pixel_format,
                                task.width, task.height, dst_pixel_format,
                                SWS_FAST_BILINEAR | SWS_ACCURATE_RND , NULL, NULL, NULL);
  // change the range of input data by first reading the current color space and then setting it's range as yuvj.
  // sws_getColorspaceDetails(swsCtx, NULL, &srcRange, NULL, &dstRange, &brightness, &contrast, &saturation);
  // sws_getColorspaceDetails(swsCtx, (int**)&dummy, dummy, (int**)&dummy, &dstRange, &brightness, &contrast, &saturation);
  coefs = sws_getCoefficients(SWS_CS_DEFAULT);
  // sws_setColorspaceDetails(swsCtx, coefs, srcRange, coefs, 0, brightness, contrast, saturation);
  sws_setColorspaceDetails(swsCtx, coefs, srcRange, coefs, 0, 0, 65535, 65535);
  
  sws_scale(swsCtx, frame->data, frame->linesize, 0, frame->height, task.yuv_data, dst_linesize);
  // av_free_packet(&packet);
  av_freep(&frame);
  sws_freeContext(swsCtx);
  avformat_close_input(&m_avfmt_ctx);

  return ret;
}

void * ImageToYuv::transProc(void *pVoid)
{
  ImageToYuv *pProc = (ImageToYuv*)pVoid;
  int ret = 0;
  int firstFrame =1;
  auto startTime=chrono::steady_clock::now();
  int frames_counter=0;
  while(pProc->m_bActive)
  {
    if(pProc->m_task.empty())
    {
      usleep(1);
      continue;
    }
    if(firstFrame)
    {
      startTime = chrono::steady_clock::now();
      firstFrame = 0;
      //printf("Taks : %d,starttime=%f\n", pProc->m_taskID,(double)tStart);
    }
    string task = pProc->m_task.front();
    pProc->m_task.pop_front();
    const char * image = task.c_str();
    Encode_Task enc_task;
    if(image!=nullptr&&image[0]!=0)
    {
      enc_task.file_name = new char [512];     
      const char * name = strrchr(image,'/');
      sprintf(enc_task.file_name ,"%s%s.heic",pProc->m_output_path,name);
      enc_task.yuv_data = new unsigned char *[4]{0};
      ret = pProc->imageToYuv(image,enc_task);
      if(ret <0 || enc_task.yuv_data[0]==nullptr)
      {
        printf("%s to yuv failed\n",image);
        continue;
      }
      frames_counter++;
       pProc->m_pHeicProc->AddTask(enc_task);
    }
    else
    {
#if DEBUG
      printf("Image to Yuv %d EOF received\n",pProc->m_taskID);
#endif
      pProc->m_bActive = false;
      pProc->m_endCb(pProc->m_taskID);
    }
   
  }
  
#if DEBUG
  auto endTime = chrono::steady_clock::now();
  auto diff = endTime - startTime;
  printf(">>>>> duration=%f s\n", chrono::duration<double,milli>(diff).count()/1000);
  if (chrono::duration<double,milli>(diff).count()/1000)
  {
    printf("Image to Yuv FPS=%.2f frames per second in %d total\n", frames_counter/(chrono::duration<double,milli>(diff).count()/1000),frames_counter);
  }
#endif
  //printf("Processing done [%d] packet on CPU %d\n",pProc->m_hevc_enc.m_sentCnt,sched_getcpu());
  printf("Yuv Image processing done on task [%d]\n",pProc->m_taskID);
  pthread_exit(nullptr);
}

