/*
 * ImageToYuv.hpp
 *
 *  Created on: May 25, 2020
 *      Author: ray
 */


#ifndef IMAGETOYUV_HPP_
#define IMAGETOYUV_HPP_
#include <pthread.h>
#include <list>
extern "C"
{
#include <time.h>
#include <libavformat/avformat.h>
#include <libswscale/swscale.h>
#include <libavutil/imgutils.h>
//#include <system/types.h>

}
#include "common_def.hpp"

class YuvToHeic;

class ImageToYuv {
public:
	ImageToYuv();
	virtual ~ImageToYuv();
	
		/**
	 * Add transfer file task into list
	 * @pSrc: image file path to be converted.
	 * @return: 0 is successful,none-zero is failed.
	 */
	int AddTask(const char*pSrc);
  /**
	 * Setup codec parameter.
	 * @param: codec parameters.
   * @type: input image type
   * @type: yuv to heic processor handle.
	 * @return: 0 is successful,none-zero is failed.
   * 
	 */
  int Setup(Codec_Param &init_param,SOURCETYPE source,YuvToHeic *);
  /**
	 * Send End of task to processor.
	 */
	void SendEOF();
	/**
	 * Start processing task, launch processing thread
	 * @taskID: task identify.
	 * @bindCore: bind processing thread to CPU core number.
	 * @return: 0 is successful,none-zero is failed.
	 */
	int Start(int taskID,void (*callback)(int), bool bindCore=false);

  	/**
	 * Stop processing thread
	 * @wait, wait tasks to finish, default is to wait.
	 * @return: 0 is successful,none-zero is failed.
	 */
	int Stop(bool wait=true);

protected:
	/**
	 * Thread function,which process image from jpeg,bmp,png to yuv.
	 * @pCodec: this pointer.
	 */
  static void* transProc(void *pCodec);

  /**
	 * Convert image to yuv.
	 * @image: input image path.
	 * @dst_data: memory to store yuv data.
	 * @return: 0 is successful,none-zero is failed.
	 */
	int imageToYuv(const char * image,Encode_Task & task);

	/**
	 * Setup codec parameter.
	 * @param: codec parameters.
	 * @type: input image type
	 * @return: 0 is successful,none-zero is failed.
	 */
	int Setup(Codec_Param &param,SOURCETYPE type);
protected:
//Thread running flag.
  bool m_bActive = false;
   // Scaler flag
  bool m_bScaler = false;
  // Scaler width
  int m_width = 0;
  // Scaler height
  int m_height = 0;
  //Thread handle.
  pthread_t m_thread;
  //task list. FIFO.
  list<string> m_task;
  //AV format context.
  AVFormatContext *m_avfmt_ctx = nullptr;
  //AV input formate pointer.
  AVInputFormat *m_in_fmt=nullptr;
  // First frame flag.
  int m_bFirstFrame = 1;
  //Output file folder.
  char m_output_path[512]={0};
  //Processing task ID.
  int m_taskID = -1;
  //Source file type.
  SOURCETYPE m_in_type = UN_SUPPORTED;
  // Yuv to Heic processor instance.
  YuvToHeic * m_pHeicProc = nullptr;
    //Task status notification callback.
  void (*m_endCb)(int)=nullptr;
};

#endif /* IMAGETOYUV_HPP_ */
