/*
 * YuvToHeic.cpp
 *
 *  Created on: Feb 20, 2020
 *      Author: root
 */
#include <string>
#include <chrono>
#include "YuvToHeic.hpp"

YuvToHeic::YuvToHeic() {
  // TODO Auto-generated constructor stub
  pthread_rwlock_init(&m_rwlock,nullptr);
}

YuvToHeic::~YuvToHeic() {
  // TODO Auto-generated destructor stub
  pthread_rwlock_destroy(&m_rwlock);
}

int YuvToHeic::AddTask(Encode_Task  task)
{
  pthread_rwlock_wrlock(&m_rwlock);
  m_task.push_back(task);
  pthread_rwlock_unlock(&m_rwlock);
  return 0;
}

void YuvToHeic::SendEOF()
{

  //  m_hevc_enc.allocateBuffer(nullptr,0);
  //  return m_hevc_enc.sendData();
  m_task.push_back(Encode_Task());
}

void YuvToHeic::SetEncodeParam(const char * key,const char * value)
{
  m_hevc_enc.setParam(key, value);
}

int YuvToHeic::Start(int taskID,void (*callback)(int), bool bindCore)
{
  m_bActive = true;
  int rc = NI_RETCODE_SUCCESS;
  //spawn a thread for decoding frame to yuv + sending yuv to HW encoder
  rc = pthread_create(&m_thread,NULL,&YuvToHeic::transProc,this);
  if(rc!=0)
  {
    printf("create receive data thread failed: %d\n",rc);
    m_bActive = false;
    return NI_RETCODE_FAILURE;
  }
  else
  {
    //printf("create receive data thread : %llu\n",m_thread);
  }
  if(bindCore)
  {
    cpu_set_t cpuset;
    CPU_ZERO(&cpuset);
    CPU_SET(taskID, &cpuset);
    rc = pthread_setaffinity_np(m_thread,sizeof(cpu_set_t), &cpuset);
    if (rc != 0) {
      printf("Error calling pthread_setaffinity_np: %d\n" ,rc );
    }
    else
    {
      //printf("Binding thread %llu to CPU %d\n",m_thread,taskID);
    }
  }
  //spawn a thread to receive compressed the 265 from encoder and put it in HEIF container
  m_hevc_enc.startRecv(bindCore,taskID);
  m_taskID= taskID;
  m_endCb = callback;

  return rc;
}

int YuvToHeic::Setup(Codec_Param &init_param)
{
  int ret = NI_RETCODE_SUCCESS;
  m_hevc_enc.m_qp = init_param.qp;
  ret = m_hevc_enc.init((ni_codec_format_t)init_param.mode,init_param.hevc_file,init_param.width,init_param.height,init_param.fps,init_param.bitrate);
  m_hevc_enc.assignDevice(init_param.hw_id);
  setXcoderParam();
  strcpy(m_output_path,init_param.dest_path);
  m_hevc_enc.openSession();
  return ret;
}

int YuvToHeic::Stop(bool wait)
{
  int ret = NI_RETCODE_SUCCESS;
  if(!m_bActive)
  {
    return ret;
  }
  if(!wait)
  {
    m_bActive = false;
    pthread_cancel(m_thread);
  }
  void * status;
  //printf("wait receive data thread to finish\n");
  ret = pthread_join(m_thread,&status);
  if(ret!=0)
  {
    printf("wait receive data thread failed: %d\n",ret);
    ret = NI_RETCODE_FAILURE;
  }
  //printf("stop thread :%llu\n",m_thread);
  m_bActive = false;

  return ret;
}

int YuvToHeic::Clean()
{
  m_hevc_enc.stopRecv();
  m_hevc_enc.closeSession();
  m_hevc_enc.removeDevice();
  remove(m_hevc_enc.m_dest.getFilePath());
  return 0;
}

void YuvToHeic::setXcoderParam()
{
  m_hevc_enc.setParam(NI_ENC_PARAM_GOP_PRESET_IDX, "1"); // encode intra frames only
  m_hevc_enc.setParam(NI_ENC_PARAM_FORCED_HEADER_ENABLE,std::to_string(m_hevc_enc.m_repeated_header).c_str());
  m_hevc_enc.setParam(NI_ENC_PARAM_FORCE_FRAME_TYPE, "0"); // only IDR frames
  m_hevc_enc.setParam(NI_ENC_PARAM_ENABLE_RATE_CONTROL,"0"); // no rate control we use QP control
  m_hevc_enc.setParam(NI_ENC_PARAM_INTRA_QP,std::to_string(m_hevc_enc.m_qp).c_str()); // use MB QP from command line
}

int YuvToHeic::resolutionChanged(int width,int height)
{
  int ret = 0;
  if (m_hevc_enc.m_sentCnt > 0)
  {
    m_hevc_enc.allocateBuffer(nullptr, 0);
    ret = m_hevc_enc.sendData();
    ret =m_hevc_enc.stopRecv();
  }
  else
  {
    ret = m_hevc_enc.stopRecv(false);
  }

  ret = m_hevc_enc.resetSession();
  if (ret < 0)
  {
    return NI_RETCODE_FAILURE;
  }
  m_hevc_enc.init_encode_default_param(width, height, m_hevc_enc.m_fps, m_hevc_enc.m_bitrate);
  setXcoderParam();
  m_hevc_enc.openSession();
  ret = m_hevc_enc.startRecv();
  printf("Task[%d] resolution change done \n",m_taskID);

  return ret;
}


// int YuvToHeic::imageToYuv(const char * image,unsigned char ** dst_data)
// {
//   AVCodecContext *dec_ctx= NULL;
//   int ret = 0;
//   AVPacket packet = { 0 };
//   AVFrame *frame = NULL;
//   AVStream *st;
//   enum AVMediaType type;
//   unsigned int stream_index;
//   unsigned int i;
//   int got_frame;

//   ret = avformat_open_input(&m_avfmt_ctx, image, m_in_fmt, NULL);
//   if(ret)
//   {
//     printf("error opening file\n");
//     return NI_RETCODE_FAILURE;
//   }
//   ret = avformat_find_stream_info(m_avfmt_ctx, NULL);
//   if(m_avfmt_ctx->streams[0]->codecpar == NULL)
//   {
//     printf("codec par stream 0 is null \n");
//   }
//   if(ret)
//   {
//     printf("error finding stream info\n");
//     return NI_RETCODE_FAILURE;
//   }
//   int video_stream_idx = 0;
//   dec_ctx = m_avfmt_ctx->streams[video_stream_idx]->codec;
//   AVCodecParameters* cdc_params = m_avfmt_ctx->streams[video_stream_idx]->codecpar;
//   AVCodec* codec = avcodec_find_decoder(cdc_params->codec_id);
//   ret = avcodec_open2(dec_ctx, codec, NULL);
//   if (ret < 0)
//   {
//     printf("Error opening decoder!\n");
//     return NI_RETCODE_FAILURE;
//   }
//   #if DEBUG
//   av_dump_format(ifmt_ctx, 0, filename, 0);

//   printf("dec context allocated \n");
//   #endif
//   av_init_packet(&packet);
//   ret = av_read_frame(m_avfmt_ctx, &packet);
//   if (ret < 0)
//   {
//     printf("Error reading packet!\n");
//     return NI_RETCODE_FAILURE;
//   }
//   frame = av_frame_alloc();
//   if (!frame)
//   {
//     printf("error allocating frame\n");
//     return NI_RETCODE_FAILURE;
//   }
//   ret = avcodec_decode_video2(dec_ctx, frame,&got_frame, &packet);
//   if (ret < 0)
//   {
//     printf("Error decoding packet!\n");
//     return NI_RETCODE_FAILURE;
//   }
//   SwsContext *swsCtx = NULL;
//   int width = m_enc_width==0? dec_ctx->width:m_enc_width;
//   int height = m_enc_height==0? dec_ctx->height:m_enc_height;

//   AVPixelFormat dst_pixel_format = AV_PIX_FMT_YUV420P;
//   int dst_linesize[4];
//   ret = av_image_alloc(dst_data, dst_linesize, width, height,dst_pixel_format,1 );
//   if (ret < 0)
//   {
//     printf("Error allocating dest image!\n");
//     return NI_RETCODE_FAILURE;
//   }
//   // remove deprecated av formats
//   if (dec_ctx->pix_fmt == AV_PIX_FMT_YUVJ420P)
//   {
//     dec_ctx->pix_fmt = AV_PIX_FMT_YUV420P;
//   }
//   else if (dec_ctx->pix_fmt == AV_PIX_FMT_YUVJ422P)
//   {
//     dec_ctx->pix_fmt = AV_PIX_FMT_YUV422P;
//   }
//   else if (dec_ctx->pix_fmt == AV_PIX_FMT_YUVJ444P)
//   {
//     dec_ctx->pix_fmt = AV_PIX_FMT_YUV444P;
//   }
//   // convert any input format to YUV420 using libav swscaler
//   swsCtx = sws_getCachedContext ( swsCtx, dec_ctx->width, dec_ctx->height,
//     dec_ctx->pix_fmt, width, height, AV_PIX_FMT_YUV420P,
//   SWS_FAST_BILINEAR | SWS_ACCURATE_RND , NULL, NULL, NULL );
//   sws_scale ( swsCtx, frame->data, frame->linesize, 0,frame->height, dst_data, dst_linesize );

//   // check if currently decoded resolution is same as previous encoded frame
//   if ((width != m_hevc_enc.m_width) || (height != m_hevc_enc.m_height))
//   {
//     printf("****Task[%d] Resolution change from %dx%d to %dx%d \n",m_taskID, m_hevc_enc.m_width, m_hevc_enc.m_height, width, height);
//     ret= resolutionChanged(dec_ctx->width,dec_ctx->height);
//   }

//   //av_free_packet(&packet);
//   av_freep(&frame);
//   sws_freeContext(swsCtx);
//   avformat_close_input(&m_avfmt_ctx);

//   return ret;
// }

int YuvToHeic::yuvTo265(const char * out,unsigned char ** dst_data)
{
  int ret = NI_RETCODE_SUCCESS;
  if(out !=nullptr)
  {
    m_hevc_enc.addHEIFFile(out);
    ret = m_hevc_enc.allocateBuffer(dst_data,m_hevc_enc.m_frameSize);
    if (ret < 0)
    {
      printf("Error allocating buffer! status = %d\n", ret);
    }
  }
  else
  {
    m_hevc_enc.allocateBuffer(nullptr,0);
  }
  //send YUV to encoder
  ret =m_hevc_enc.sendData();
  return ret;
}

void * YuvToHeic::transProc(void *pVoid)
{
  YuvToHeic *pProc = (YuvToHeic*)pVoid;
  int ret = NI_RETCODE_SUCCESS;
  int firstFrame =1;
  auto startTime=chrono::steady_clock::now();
  int frames_counter=0;
  while(pProc->m_bActive&& pProc->m_hevc_enc.m_eos_received==0)
  {
    if(pProc->m_task.empty())
    {
      usleep(1);
      continue;
    }
    if(firstFrame)
    {
      startTime = chrono::steady_clock::now();
      firstFrame = 0;
      //printf("Taks : %d,starttime=%f\n", pProc->m_taskID,(double)tStart);
    }
    
    pthread_rwlock_rdlock(&(pProc->m_rwlock));
    Encode_Task  task = std::move(pProc->m_task.front());
    pProc->m_task.pop_front();
    pthread_rwlock_unlock(&(pProc->m_rwlock));
    const char * heic_file = task.file_name;
    if(heic_file!=nullptr&&heic_file[0]!=0)
    {
      if ((task.width != pProc->m_hevc_enc.m_width) || (task.height != pProc->m_hevc_enc.m_height))
      {
        printf("****Sesssion Task[%d] Resolution change from %dx%d to %dx%d \n",pProc->m_taskID, pProc->m_hevc_enc.m_width, pProc->m_hevc_enc.m_height, task.width, task.height);
        ret= pProc->resolutionChanged(task.width ,task.height); 
      }
      ret = pProc->yuvTo265(heic_file, task.yuv_data);
      if (ret < 0)
      {
        printf("Error encoding data!\n");
      }
//      printf("Yuv to Heic processing time %fs\n",((double)(clock()-start))/CLOCKS_PER_SEC);
      av_freep(task.yuv_data);
      delete [] (task.yuv_data);
      delete [] (task.file_name);
      frames_counter++;
    }
    else
    {
#if DEBUG
      printf("End of task %d\n",pProc->m_taskID);
#endif
      if(frames_counter==0)
      {
        pProc->m_hevc_enc.stopRecv(false);
        pProc->m_bActive = false;
      }
      else
      {
        pProc->yuvTo265(nullptr,nullptr);

      }
    }
  }
#if DEBUG
  auto endTime = chrono::steady_clock::now();
  auto diff = endTime - startTime;
  printf("Yuv to Heic >>>>> duration=%f s\n", chrono::duration<double,milli>(diff).count()/1000);
  if (chrono::duration<double,milli>(diff).count()/1000)
  {
    printf("Yuv to Heic FPS=%.2f frames per second, %d in total \n", frames_counter/(chrono::duration<double,milli>(diff).count()/1000),frames_counter);
  }
#endif
  if(pProc->m_endCb)
  {
    pProc->m_endCb(pProc->m_taskID);
  }
  //printf("Processing done [%d] packet on CPU %d\n",pProc->m_hevc_enc.m_sentCnt,sched_getcpu());
  printf("Yuv to Heic processing done on task [%d]\n",pProc->m_taskID);
  pthread_exit(nullptr);
}

