	/*
 * YuvToHeic.hpp
 *
 *  Created on: Feb 20, 2020
 *      Author: ray
 */

#ifndef YUVTOHEIC_HPP_
#define YUVTOHEIC_HPP_
#include <pthread.h>
#include <list>
#include "Encoder.hpp"
#include "common_def.hpp"
extern "C"
{
#include <time.h>
#include <libavformat/avformat.h>
#include <libswscale/swscale.h>
#include <libavutil/imgutils.h>
//#include <system/types.h>

}


class YuvToHeic
{
public:
	YuvToHeic();
	virtual ~YuvToHeic();
	/**
	 * Add transfer file task into list
	 * @pSrc: image file path to be converted.
	 * @return: 0 is successful,none-zero is failed.
	 */
	int AddTask(Encode_Task task);
	/**
	 * Send End of task to processor.
	 */
	void SendEOF();
	/**
	 * Start processing task, launch processing thread
	 * @taskID: task identify.
	 * @callback: callback function to notify all tasks assigned to processor have been done.
	 * @bindCore: bind processing thread to CPU core number.
	 * @return: 0 is successful,none-zero is failed.
	 */
	int Start(int taskID,void (*callback)(int) , bool bindCore=false);
	/**
	 * Setup codec parameter.
	 * @param: codec parameters.
	 * @return: 0 is successful,none-zero is failed.
	 */
	int Setup(Codec_Param &param);
	/**
	 * Stop processing thread
	 * @wait, wait tasks to finish, default is to wait.
	 * @return: 0 is successful,none-zero is failed.
	 */
	int Stop(bool wait=true);
	/**
	 * Set HEVC encoder parameters,refer to all libxcoder parameters.
	 *
	 */
	void SetEncodeParam(const char * key,const char * value);
	/**
	 * clean up all allocated resources, including release libxcoder resource.
	 * @return: 0 is successful,none-zero is failed.
	 */
	int Clean();

protected:
	/**
	 * Thread function,which process image from jpeg,bmp,png to yuv, then send to libxcoder.
	 * @pCodec: this pointer.
	 */
	static void* transProc(void *pCodec);
	/**
	 * Send yuv data to libxcoder.
	 * @out: output heic file name.
	 * @dst_data: yuv data.
	 * @return: 0 is successful,none-zero is failed.
	 */
	int yuvTo265(const char * out,unsigned char ** dst_data);
	/**
	 * Process different resolution, re-open libxcoder session.
	 * @width: new picture width.
	 * @height: new picture height.
	 * @return: 0 is successful,none-zero is failed.
	 */
  int resolutionChanged(int width, int height);
	/**
	 * Set libxcoder parameters.
	 */
  void setXcoderParam();
protected:
  //Thread running flag.
  bool m_bActive=false;
  //Initial flag.
  bool m_bInit = false;
  //Thread handle.
  pthread_t m_thread;
  //YUV to HEVC encoder instance.
  Encoder m_hevc_enc;
  //task list. FIFO.
  list<Encode_Task> m_task;
	//task queue lock.
	pthread_rwlock_t m_rwlock;
  // //AV format context.
  // AVFormatContext *m_avfmt_ctx = nullptr;
  // //AV input formate pointer.
  // AVInputFormat *m_in_fmt=nullptr;
  // First frame flag.
  int m_bFirstFrame = 1;
  //Output file folder.
  char m_output_path[512]={0};
  //Processing task ID.
  int m_taskID = -1;
  // //Source file type.
  // SOURCETYPE m_in_type = UN_SUPPORTED;
  //Task status notification callback.
  void (*m_endCb)(int)=nullptr;

	// //target width.
	// int m_enc_width = 0;
	// //target height.
	// int m_enc_height = 0;

};

#endif /* YuvToHeic_HPP_ */
