/*
 * main.cpp
 *
 *  Created on: Jan 31, 2020
 *      Author: ray
 */
#include <stdio.h>

#include <iostream>
#include <unistd.h>
#include <stdlib.h>
#include <string>
#include <string.h>
#include <pthread.h>
#include <thread>
#include <semaphore.h>
#include <sys/stat.h>
#include "Encoder.hpp"
extern "C"
{
#include <time.h>
#include <libavformat/avformat.h>
#include <libswscale/swscale.h>
#include <libavutil/imgutils.h>

}
#include <dirent.h>
#include "ICodec.hpp"
#include "YuvToHeic.hpp"
#include "ImageToYuv.hpp"
#include <chrono>
#include <ni_rsrc_api.h>
#include <ni_util.h>
#include "common_def.hpp"
//files manip includes


using namespace std;

int session_size;
static YuvToHeic *HeicProcList;

// struct task_info
// {
//   task_info (int id,SOURCETYPE type)
//   {
//     task_id = id;
//     task_type =type;
//   };
//   bool operator == (const task_info & rhs)
//   {
//     if(rhs.task_id == task_id && rhs.task_type==task_type)
//       return true;
//     else
//       return false;
//   }
//   int task_id =-1;
//   int task_type=-1;
// };
unsigned int HEIC_TASK_FLAG = 0;
list<int> yuv_task_list;
sem_t task_complete_sem;

void EndYuvToHeic(int id)
{
  HEIC_TASK_FLAG &= ~(1 << id); 
#if DEBUG
  printf("++++++++++NI image converter Task %d ended, task flag 0x%x\n",id,HEIC_TASK_FLAG);
#endif
  if(HEIC_TASK_FLAG==0)
  {
    sem_post(&task_complete_sem);
  } 
};

void EndImageToYuv(int id)
{
  yuv_task_list.remove(id);
#if DEBUG
  printf("++++++++++Image to Yuv converter Task %d ended, task left %d  \n",id,yuv_task_list.size());
#endif
  if(yuv_task_list.empty())
  {
    printf("Image to Yuv task done\n");
    for(int i = 0; i < session_size; i++)
    {
      HeicProcList[i].SendEOF();
    }
  } 
};

int InitImageToHeic(Codec_Param &init_param)
{
  int ret = 0;
  for(int i = 0; i < session_size; i++)
  {
      sprintf(init_param.hevc_file,"jpeg2h265_thread_%d.h265",i);
      ret = HeicProcList[i].Setup(init_param);
      if(ret < 0)
      {
        printf("YuvToHeic %d setup failed\n",i);
        break;
      }
      ret = HeicProcList[i].Start(i,EndYuvToHeic);
      if(ret < 0)
      {
        printf("YuvToHeic %d start failed\n",i);
        break;
      }
      HEIC_TASK_FLAG |= 1 << i;
  }
  return ret;
}

int CleanImageToHeic()
{
  int ret = 0;
  for(int i = 0; i < session_size; i++)
  { 
      ret = HeicProcList[i].Stop();
      if(ret < 0)
      {
        printf("YuvToHeic %d stop failed\n",i);
      }
      ret = HeicProcList[i].Clean();
      if(ret < 0)
      {
        printf("YuvToHeic %d clean failed\n",i);
      }
  }
  delete [] HeicProcList;
  return ret;
}

void print_help_text()
{
  printf("-------- NETINT_HEIC_converter --------\n"
          "This program uses the Netint T408/T432 video transcoder to convert images of\n"
          "various formats into the HEIC format.\n"
          "Usage: ./NETINT_HEIC_converter <input_folder> <output_folder> <QP> [OPTIONS]\n"
          "\n"
          "Arguments:\n"
          "    -s WxH    Resize images to resolution WxH. Default: OFF\n"
          "    -n N      Use N parallel encoding sessions. Default: 4*num_encoders\n"
          "    -r N      Read input folder N times (for benchmarking). Default: 1\n"
          "    -e I      Use encoder at index I. Default: let libxcoder decide\n"
          "    -l N      Use libxcoder logging level N.\n"
          "              Options: [none, fatal, error, info, debug, trace]. Default: info\n"
          "    -h        Display help text.\n");
}

int main(int argc, char ** argv)
{
  int opt, device_cnt;      // for arg parsing
  char *n;      // for parsing width and height from -s
  char *input_path;
  int input_loops = 1;
  int ret = 0;
  int frames_counter = 0;
  Codec_Param init_param;
  // default encoder frame size init
  init_param.width = 1920;
  init_param.height = 1080;

  // set default number of encoding sessions to use as 4*num_of_enc
  ni_device_info_t p_device_info[MAX_DEVICE_CNT] = {0};
  ni_retcode_t ni_ret = ni_rsrc_list_devices(NI_DEVICE_TYPE_ENCODER, p_device_info, &device_cnt);
  session_size = device_cnt * 4;

  // arg handling
  if (argc < 4)
  {
    print_help_text();
    exit(-1);
  }
  input_path = argv[1];
  strcpy(init_param.dest_path, argv[2]);
  init_param.qp = atoi(argv[3]);

  while ((opt = getopt(argc, argv, "s:n:r:e:l:h")) != -1)
  {
    switch (opt)
    {
    case 's':
      init_param.width = (int) strtol(optarg, &n, 10);
      if (*n != 'x')
      {
        fprintf(stderr, "ERROR - rescale size arg not of format [width]x[height]\n");
        exit(-1);
      }
      init_param.height = atoi(n + 1);
      break;
    case 'n':
      session_size = atoi(optarg);
      break;
    case 'r':
      input_loops = atoi(optarg);
      break;
    case 'e':
      init_param.hw_id = atoi(optarg);
      for (int i = 0; i < device_cnt; i++)
      {
        if (p_device_info[i].module_id == init_param.hw_id)
        {
          strcpy(init_param.dev_name, p_device_info[i].dev_name);
          strcpy(init_param.blk_name, p_device_info[i].blk_name);
          break;
        }
      }
      if (init_param.dev_name[0] != 0)
      {
        break;
      }
      else
      {
        fprintf(stderr, "ERROR - could not find encoder of index %d\n", init_param.hw_id);
        exit(-1);
      }
    case 'l':
      if (!strcmp(optarg, "none")) {
        ni_log_set_level(NI_LOG_NONE);
      } else if (!strcmp(optarg, "fatal")) {
        ni_log_set_level(NI_LOG_FATAL);
      } else if (!strcmp(optarg, "error")) {
        ni_log_set_level(NI_LOG_ERROR);
      } else if (!strcmp(optarg, "info")) {
        ni_log_set_level(NI_LOG_INFO);
      } else if (!strcmp(optarg, "debug")) {
        ni_log_set_level(NI_LOG_DEBUG);
      } else if (!strcmp(optarg, "trace")) {
        ni_log_set_level(NI_LOG_TRACE);
      } else {
        fprintf(stderr, "unknown log level selected: %s", optarg);
        exit(-1);
      }
      break;
    case 'h':
      print_help_text();
      exit(0);
    default:
      print_help_text();
      exit(-1);
    }
  }

  // arg checking
  struct stat stat_buffer;
  if (stat (input_path, &stat_buffer) != 0)
  {
    fprintf(stderr, "ERROR - input folder does not exist\n");
    exit(-1);
  }
  if (stat (init_param.dest_path, &stat_buffer) != 0)
  {
    fprintf(stderr, "ERROR - output folder does not exist\n");
    exit(-1);
  }
  if ((init_param.qp < 0) || (init_param.qp > 51))
  {
    fprintf(stderr, "ERROR - QP output range 0-51\n");
    exit(-1);
  }
  if ((init_param.width < 0) || (init_param.width > 8192))
  {
    fprintf(stderr, "ERROR - output resolution scaling width out of range 0-8192\n");
    exit(-1);
  }
  if ((init_param.height < 0) || (init_param.height > 8192))
  {
    fprintf(stderr, "ERROR - output resolution scaling height out of range 0-8192\n");
    exit(-1);
  }
  if ((session_size < 1))
  {
    fprintf(stderr, "ERROR - parallel encoders session count must be >=1\n");
    exit(-1);
  }
  if ((input_loops < 1))
  {
    fprintf(stderr, "ERROR - number of times to read input folder must be >=1\n");
    exit(-1);
  }

  HeicProcList = new YuvToHeic[session_size];

  sem_init(&task_complete_sem, 0, 0);
  init_param.fps = 30; // just an init value
  init_param.mode = 1; // HEVC enc
  init_param.bitrate = 10000000; // to be a score of quality later
  
  if (InitImageToHeic(init_param) < 0)
  {
    fprintf(stderr, "Start NI image converter failed");
    delete [] HeicProcList;
    exit(-1);
  }

  int cpu_core = std::thread::hardware_concurrency();
  bool bindCore = true;

  printf("%d concurrent threads are supported, max %d threads will be launched.\n", std::thread::hardware_concurrency(), cpu_core);

  ImageToYuv ** jpg_processors = new ImageToYuv*[cpu_core]{0};
  ImageToYuv ** bmp_processors = new ImageToYuv*[cpu_core]{0};
  ImageToYuv ** png_processors = new ImageToYuv*[cpu_core]{0};
  ImageToYuv ** processors = nullptr;
  printf("\033[1;36m====== application starts.\033[0m\n");
  auto startTime = chrono::steady_clock::now();
  DIR* dirFile = opendir(input_path);
  if (dirFile)
  {
    struct dirent* hFile;
    errno = 0;
    char inputFullPath[512] = {0};
    for (int i = 0; i < input_loops; i++)
    {
      // read all files from input with supported extensions
      while (hFile = readdir(dirFile))
      {
        if (!strcmp( hFile->d_name, ".") || !strcmp( hFile->d_name, "..")) continue;
        if (hFile->d_type != DT_REG)
        {
          printf("Skip not regular file %s\n",hFile->d_name);
          continue;
        }
        const char * extension = strrchr(hFile->d_name,'.');
        SOURCETYPE type = UN_SUPPORTED;
        if (!strcmp( extension, ".jpg" ) || !strcmp( extension, ".jpeg" ) || !strcmp( extension, ".JPG" ))
        {
          type = JPEG_FILE;
          processors = jpg_processors;
        }
        else if (!strcmp( extension, ".bmp" ))
        {
          type = BMP_FILE;
          processors = bmp_processors;
        }
        else if (!strcmp( extension, ".png" ))
        {
          type = PNG_FILE;
          processors = png_processors;
        }
        else
          continue;
        sprintf(inputFullPath, "%s/%s", input_path, hFile->d_name);
        int idx = frames_counter%cpu_core;
        if(processors[idx]==nullptr)
        {
          processors[idx] = new ImageToYuv;
          processors[idx]->Setup(init_param,type,&(HeicProcList[(idx+1)%session_size]));
          processors[idx]->Start(idx,EndImageToYuv,bindCore);
          yuv_task_list.push_back(idx);
        }
        processors[idx]->AddTask(inputFullPath);
        frames_counter++;
      }
      rewinddir(dirFile);
    }
    closedir(dirFile);
  }

  printf("Totally imported %d pictures\n",frames_counter);
  for(int i =0 ;i<cpu_core;i++)
  {
    if(jpg_processors[i]!=nullptr)
      jpg_processors[i]->SendEOF();
    if(bmp_processors[i]!=nullptr)
      bmp_processors[i]->SendEOF();
    if(png_processors[i]!=nullptr)
      png_processors[i]->SendEOF();
  }

  sem_wait(&task_complete_sem);
  auto endTime = chrono::steady_clock::now();
  auto diff = endTime - startTime;
  printf(">>>>> duration = %f s for a total number of frames = %d\n", chrono::duration<double,milli>(diff).count()/1000, frames_counter);
  if (chrono::duration<double,milli>(diff).count()/1000)
  {
    printf("\033[1;36mFPS=%f frames per second \033[0m\n", frames_counter/(chrono::duration<double,milli>(diff).count()/1000));
  }

  //mem clean
  for(int i =0 ;i<cpu_core;i++)
  {
    if(jpg_processors[i]!=nullptr)
    {
      jpg_processors[i]->Stop();
      delete jpg_processors[i];
      jpg_processors[i]=nullptr;
    }
    if(bmp_processors[i]!=nullptr)
    {
      bmp_processors[i]->Stop();
      delete bmp_processors[i];
      bmp_processors[i]=nullptr;
    }
    if(png_processors[i]!=nullptr)
    {
      png_processors[i]->Stop();
      delete png_processors[i];
      png_processors[i]=nullptr;
    }
  }
  delete []jpg_processors;
  delete []bmp_processors;
  delete []png_processors;
  CleanImageToHeic();
  // ICodec::CloseDevice(init_param.dev_name,init_param.blk_name);
  sem_destroy(&task_complete_sem);
  printf("\033[1;36m====== application ends. \033[0m\n");
}

