/*!****************************************************************************
*
* Copyright (C)  2018 NETINT Technologies. 
*
* Permission to use, copy, modify, and/or distribute this software for any 
* purpose with or without fee is hereby granted.
*
*******************************************************************************/

/*!*****************************************************************************
*  \file   ni_av_codec_logan.h
*
*  \brief  NETINT audio/video related utility functions
*
*******************************************************************************/

#pragma once

#ifdef __cplusplus
extern "C"
{
#endif

#include "ni_device_api_logan.h"

// libxcoder API related definitions

#ifndef QUADRA

//Map the gopPresetIdx to gopSize, for customer set this value to -1
//customer_gop_size can find in ni_h265_enable_params_t struct
static int g_map_preset_to_gopsize[10] = {-1, 1, 1, 1, 2, 4, 4, 4, 8, 1};

#define NI_NUM_PIXEL_ASPECT_RATIO 17
static const ni_rational_t ni_h264_pixel_aspect_list[NI_NUM_PIXEL_ASPECT_RATIO]=
{
  {   0,  1 },
  {   1,  1 },
  {  12, 11 },
  {  10, 11 },
  {  16, 11 },
  {  40, 33 },
  {  24, 11 },
  {  20, 11 },
  {  32, 11 },
  {  80, 33 },
  {  18, 11 },
  {  15, 11 },
  {  64, 33 },
  { 160, 99 },
  {   4,  3 },
  {   3,  2 },
  {   2,  1 },
};

typedef enum _ni_h264_sei_type_t
{
  NI_H264_SEI_TYPE_BUFFERING_PERIOD       = 0,   // buffering period (H.264, D.1.1)
  NI_H264_SEI_TYPE_PIC_TIMING             = 1,   // picture timing
  NI_H264_SEI_TYPE_PAN_SCAN_RECT          = 2,   // pan-scan rectangle
  NI_H264_SEI_TYPE_FILLER_PAYLOAD         = 3,   // filler data
  NI_H264_SEI_TYPE_USER_DATA_REGISTERED   = 4,   // registered user data as specified by Rec. ITU-T T.35
  NI_H264_SEI_TYPE_USER_DATA_UNREGISTERED = 5,   // unregistered user data
  NI_H264_SEI_TYPE_RECOVERY_POINT         = 6,   // recovery point (frame # to decoder sync)
  NI_H264_SEI_TYPE_FRAME_PACKING          = 45,  // frame packing arrangement
  NI_H264_SEI_TYPE_DISPLAY_ORIENTATION    = 47,  // display orientation
  NI_H264_SEI_TYPE_GREEN_METADATA         = 56,  // GreenMPEG information
  NI_H264_SEI_TYPE_MASTERING_DISPLAY_COLOUR_VOLUME = 137,  // mastering display properties
  NI_H264_SEI_TYPE_ALTERNATIVE_TRANSFER   = 147, // alternative transfer
} ni_h264_sei_type_t;

// pic_struct in picture timing SEI message
typedef enum _ni_h264_sei_pic_struct_t
{
  NI_H264_SEI_PIC_STRUCT_FRAME             = 0, //  0: %frame
  NI_H264_SEI_PIC_STRUCT_TOP_FIELD         = 1, //  1: top field
  NI_H264_SEI_PIC_STRUCT_BOTTOM_FIELD      = 2, //  2: bottom field
  NI_H264_SEI_PIC_STRUCT_TOP_BOTTOM        = 3, //  3: top field, bottom field, in that order
  NI_H264_SEI_PIC_STRUCT_BOTTOM_TOP        = 4, //  4: bottom field, top field, in that order
  NI_H264_SEI_PIC_STRUCT_TOP_BOTTOM_TOP    = 5, //  5: top field, bottom field, top field repeated, in that order
  NI_H264_SEI_PIC_STRUCT_BOTTOM_TOP_BOTTOM = 6, //  6: bottom field, top field, bottom field repeated, in that order
  NI_H264_SEI_PIC_STRUCT_FRAME_DOUBLING    = 7, //  7: %frame doubling
  NI_H264_SEI_PIC_STRUCT_FRAME_TRIPLING    = 8  //  8: %frame tripling
} ni_h264_sei_pic_struct_t;

/**
 * MPEG vs JPEG YUV range.
 */
typedef enum _ni_color_range_struct_t
{
    NI_COL_RANGE_UNSPECIFIED = 0,
    NI_COL_RANGE_MPEG        = 1, ///< the normal 219*2^(n-8) "MPEG" YUV ranges
    NI_COL_RANGE_JPEG        = 2, ///< the normal     2^n-1   "JPEG" YUV ranges
    NI_COL_RANGE_NB,              ///< Not part of ABI
}ni_color_range_struct_t;

/* Chromaticity coordinates of the source primaries.
 * These values match the ones defined by ISO/IEC 23001-8_2013 § 7.1.
 */
typedef enum _ni_color_primaries
{
  NI_COL_PRI_RESERVED0   = 0,
  NI_COL_PRI_BT709       = 1,  //< also ITU-R BT1361 / IEC 61966-2-4 / SMPTE RP177 Annex B
  NI_COL_PRI_UNSPECIFIED = 2,
  NI_COL_PRI_RESERVED    = 3,
  NI_COL_PRI_BT470M      = 4,  //< also FCC Title 47 Code of Federal Regulations 73.682 (a)(20)

  NI_COL_PRI_BT470BG     = 5,  //< also ITU-R BT601-6 625 / ITU-R BT1358 625 / ITU-R BT1700 625 PAL & SECAM
  NI_COL_PRI_SMPTE170M   = 6,  //< also ITU-R BT601-6 525 / ITU-R BT1358 525 / ITU-R BT1700 NTSC
  NI_COL_PRI_SMPTE240M   = 7,  //< functionally identical to above
  NI_COL_PRI_FILM        = 8,  //< colour filters using Illuminant C
  NI_COL_PRI_BT2020      = 9,  //< ITU-R BT2020
  NI_COL_PRI_SMPTE428    = 10, //< SMPTE ST 428-1 (CIE 1931 XYZ)
  NI_COL_PRI_SMPTEST428_1 = NI_COL_PRI_SMPTE428,
  NI_COL_PRI_SMPTE431    = 11, //< SMPTE ST 431-2 (2011) / DCI P3
  NI_COL_PRI_SMPTE432    = 12, //< SMPTE ST 432-1 (2010) / P3 D65 / Display P3
  NI_COL_PRI_JEDEC_P22   = 22, //< JEDEC P22 phosphors
  NI_COL_PRI_NB                //< Not part of ABI
} ni_color_primaries_t;

/* Color Transfer Characteristic.
 * These values match the ones defined by ISO/IEC 23001-8_2013 § 7.2.
 */
typedef enum _ni_color_transfer_characteristic
{
  NI_COL_TRC_RESERVED0    = 0,
  NI_COL_TRC_BT709        = 1,  //< also ITU-R BT1361
  NI_COL_TRC_UNSPECIFIED  = 2,
  NI_COL_TRC_RESERVED     = 3,
  NI_COL_TRC_GAMMA22      = 4,  //< also ITU-R BT470M / ITU-R BT1700 625 PAL & SECAM
  NI_COL_TRC_GAMMA28      = 5,  //< also ITU-R BT470BG
  NI_COL_TRC_SMPTE170M    = 6,  //< also ITU-R BT601-6 525 or 625 / ITU-R BT1358 525 or 625 / ITU-R BT1700 NTSC
  NI_COL_TRC_SMPTE240M    = 7,
  NI_COL_TRC_LINEAR       = 8,  //< "Linear transfer characteristics"
  NI_COL_TRC_LOG          = 9,  //< "Logarithmic transfer characteristic (100:1 range)"
  NI_COL_TRC_LOG_SQRT     = 10, //< "Logarithmic transfer characteristic (100 * Sqrt(10) : 1 range)"
  NI_COL_TRC_IEC61966_2_4 = 11, //< IEC 61966-2-4
  NI_COL_TRC_BT1361_ECG   = 12, //< ITU-R BT1361 Extended Colour Gamut
  NI_COL_TRC_IEC61966_2_1 = 13, //< IEC 61966-2-1 (sRGB or sYCC)
  NI_COL_TRC_BT2020_10    = 14, //< ITU-R BT2020 for 10-bit system
  NI_COL_TRC_BT2020_12    = 15, //< ITU-R BT2020 for 12-bit system
  NI_COL_TRC_SMPTE2084    = 16, //< SMPTE ST 2084 for 10-, 12-, 14- and 16-bit systems
  NI_COL_TRC_SMPTEST2084  = NI_COL_TRC_SMPTE2084,
  NI_COL_TRC_SMPTE428     = 17, //< SMPTE ST 428-1
  NI_COL_TRC_SMPTEST428_1 = NI_COL_TRC_SMPTE428,
  NI_COL_TRC_ARIB_STD_B67 = 18, //< ARIB STD-B67, known as "Hybrid log-gamma"
  NI_COL_TRC_NB                 //< Not part of ABI
} ni_color_transfer_characteristic_t;

/**
 * YUV colorspace type.
 * These values match the ones defined by ISO/IEC 23001-8_2013 § 7.3.
 */
typedef enum _ni_color_space
{
  NI_COL_SPC_RGB         = 0,  //< order of coefficients is actually GBR, also IEC 61966-2-1 (sRGB)
  NI_COL_SPC_BT709       = 1,  //< also ITU-R BT1361 / IEC 61966-2-4 xvYCC709 / SMPTE RP177 Annex B
  NI_COL_SPC_UNSPECIFIED = 2,
  NI_COL_SPC_RESERVED    = 3,
  NI_COL_SPC_FCC         = 4,  //< FCC Title 47 Code of Federal Regulations 73.682 (a)(20)
  NI_COL_SPC_BT470BG     = 5,  //< also ITU-R BT601-6 625 / ITU-R BT1358 625 / ITU-R BT1700 625 PAL & SECAM / IEC 61966-2-4 xvYCC601
  NI_COL_SPC_SMPTE170M   = 6,  //< also ITU-R BT601-6 525 / ITU-R BT1358 525 / ITU-R BT1700 NTSC
  NI_COL_SPC_SMPTE240M   = 7,  //< functionally identical to above
  NI_COL_SPC_YCGCO       = 8,  //< Used by Dirac / VC-2 and H.264 FRext, see ITU-T SG16
  NI_COL_SPC_YCOCG       = NI_COL_SPC_YCGCO,
  NI_COL_SPC_BT2020_NCL  = 9,  //< ITU-R BT2020 non-constant luminance system
  NI_COL_SPC_BT2020_CL   = 10, //< ITU-R BT2020 constant luminance system
  NI_COL_SPC_SMPTE2085   = 11, //< SMPTE 2085, Y'D'zD'x
  NI_COL_SPC_CHROMA_DERIVED_NCL = 12, //< Chromaticity-derived non-constant luminance system
  NI_COL_SPC_CHROMA_DERIVED_CL = 13, //< Chromaticity-derived constant luminance system
  NI_COL_SPC_ICTCP       = 14, //< ITU-R BT.2100-0, ICtCp
  NI_COL_SPC_NB                //< Not part of ABI
} ni_color_space_t;

// struct describing HDR10 mastering display metadata
typedef struct _ni_mastering_display_metadata
{
  // CIE 1931 xy chromaticity coords of color primaries (r, g, b order).
  ni_rational_t display_primaries[3][2];

  // CIE 1931 xy chromaticity coords of white point.
  ni_rational_t white_point[2];

  // Min luminance of mastering display (cd/m^2).
  ni_rational_t min_luminance;

  // Max luminance of mastering display (cd/m^2).
  ni_rational_t max_luminance;

  // Flag indicating whether the display primaries (and white point) are set.
  int has_primaries;

  // Flag indicating whether the luminance (min_ and max_) have been set.
  int has_luminance;
} ni_mastering_display_metadata_t;

// struct describing HDR10 Content light level
typedef struct _ni_content_light_level
{
  // Max content light level (cd/m^2).
  uint16_t max_cll;

  // Max average light level per frame (cd/m^2).
  uint16_t max_fall;
} ni_content_light_level_t;

// struct and enum for HDR10+
// Option for overlapping elliptical pixel selectors in an image.
typedef enum _ni_hdr_plus_overlap_process_option
{
  NI_HDR_PLUS_OVERLAP_PROCESS_WEIGHTED_AVERAGING = 0,
  NI_HDR_PLUS_OVERLAP_PROCESS_LAYERING = 1,
} ni_hdr_plus_overlap_process_option_t;

// struct that represents the percentile at a specific percentage in
// a distribution.
typedef struct _ni_hdr_plus_percentile
{
  // The percentage value corresponding to a specific percentile linearized
  // RGB value in the processing window in the scene. The value shall be in
  // the range of 0 to 100, inclusive.
  uint8_t percentage;

  // The linearized maxRGB value at a specific percentile in the processing
  // window in the scene. The value shall be in the range of 0 to 1, inclusive
  // and in multiples of 0.00001.
  ni_rational_t percentile;
} ni_hdr_plus_percentile_t;

// struct describing color transform parameters at a processing window in a
// dynamic metadata for SMPTE 2094-40.
typedef struct _ni_hdr_plus_color_transform_params
{
  // The relative x coordinate of the top left pixel of the processing
  // window. The value shall be in the range of 0 and 1, inclusive and
  //  in multiples of 1/(width of Picture - 1). The value 1 corresponds
  //  to the absolute coordinate of width of Picture - 1. The value for
  //  first processing window shall be 0.
  ni_rational_t window_upper_left_corner_x;

  //  The relative y coordinate of the top left pixel of the processing
  //  window. The value shall be in the range of 0 and 1, inclusive and
  //  in multiples of 1/(height of Picture - 1). The value 1 corresponds
  //  to the absolute coordinate of height of Picture - 1. The value for
  //  first processing window shall be 0.
  ni_rational_t window_upper_left_corner_y;

  //  The relative x coordinate of the bottom right pixel of the processing
  //  window. The value shall be in the range of 0 and 1, inclusive and
  //  in multiples of 1/(width of Picture - 1). The value 1 corresponds
  //  to the absolute coordinate of width of Picture - 1. The value for
  //  first processing window shall be 1.
  ni_rational_t window_lower_right_corner_x;

  //  The relative y coordinate of the bottom right pixel of the processing
  //  window. The value shall be in the range of 0 and 1, inclusive and
  //  in multiples of 1/(height of Picture - 1). The value 1 corresponds
  //  to the absolute coordinate of height of Picture - 1. The value for
  //  first processing window shall be 1.
  ni_rational_t window_lower_right_corner_y;

  //  The x coordinate of the center position of the concentric internal and
  //  external ellipses of the elliptical pixel selector in the processing
  //  window. The value shall be in the range of 0 to (width of Picture - 1),
  //  inclusive and in multiples of 1 pixel.
  uint16_t center_of_ellipse_x;

  //  The y coordinate of the center position of the concentric internal and
  //  external ellipses of the elliptical pixel selector in the processing
  //  window. The value shall be in the range of 0 to (height of Picture - 1),
  //  inclusive and in multiples of 1 pixel.
  uint16_t center_of_ellipse_y;

  //  The clockwise rotation angle in degree of arc with respect to the
  //  positive direction of the x-axis of the concentric internal and external
  //  ellipses of the elliptical pixel selector in the processing window. The
  //  value shall be in the range of 0 to 180, inclusive and in multiples of 1.
  uint8_t rotation_angle;

  //  The semi-major axis value of the internal ellipse of the elliptical pixel
  //  selector in amount of pixels in the processing window. The value shall be
  //  in the range of 1 to 65535, inclusive and in multiples of 1 pixel.
  uint16_t semimajor_axis_internal_ellipse;

  //  The semi-major axis value of the external ellipse of the elliptical pixel
  //  selector in amount of pixels in the processing window. The value
  //  shall not be less than semimajor_axis_internal_ellipse of the current
  //  processing window. The value shall be in the range of 1 to 65535,
  //  inclusive and in multiples of 1 pixel.
  uint16_t semimajor_axis_external_ellipse;

  //  The semi-minor axis value of the external ellipse of the elliptical pixel
  //  selector in amount of pixels in the processing window. The value shall be
  //  in the range of 1 to 65535, inclusive and in multiples of 1 pixel.
  uint16_t semiminor_axis_external_ellipse;

  //  Overlap process option indicates one of the two methods of combining
  //  rendered pixels in the processing window in an image with at least one
  //  elliptical pixel selector. For overlapping elliptical pixel selectors
  //  in an image, overlap_process_option shall have the same value.
  ni_hdr_plus_overlap_process_option_t overlap_process_option;

  //  The maximum of the color components of linearized RGB values in the
  //  processing window in the scene. The values should be in the range of 0 to
  //  1, inclusive and in multiples of 0.00001. maxscl[ 0 ], maxscl[ 1 ], and
  //  maxscl[ 2 ] are corresponding to R, G, B color components respectively.
  ni_rational_t maxscl[3];

  //  The average of linearized maxRGB values in the processing window in the
  //  scene. The value should be in the range of 0 to 1, inclusive and in
  //  multiples of 0.00001.
  ni_rational_t average_maxrgb;

  //  The number of linearized maxRGB values at given percentiles in the
  //  processing window in the scene. The maximum value shall be 15.
  uint8_t num_distribution_maxrgb_percentiles;

  //  The linearized maxRGB values at given percentiles in the
  //  processing window in the scene.
  ni_hdr_plus_percentile_t distribution_maxrgb[15];

  //  The fraction of selected pixels in the image that contains the brightest
  //  pixel in the scene. The value shall be in the range of 0 to 1, inclusive
  //  and in multiples of 0.001.
  ni_rational_t fraction_bright_pixels;

  //  This flag indicates that the metadata for the tone mapping function in
  //  the processing window is present (for value of 1).
  uint8_t tone_mapping_flag;

  //  The x coordinate of the separation point between the linear part and the
  //  curved part of the tone mapping function. The value shall be in the range
  //  of 0 to 1, excluding 0 and in multiples of 1/4095.
  ni_rational_t knee_point_x;

  //  The y coordinate of the separation point between the linear part and the
  //  curved part of the tone mapping function. The value shall be in the range
  //  of 0 to 1, excluding 0 and in multiples of 1/4095.
  ni_rational_t knee_point_y;

  //  The number of the intermediate anchor parameters of the tone mapping
  //  function in the processing window. The maximum value shall be 15.
  uint8_t num_bezier_curve_anchors;

  //  The intermediate anchor parameters of the tone mapping function in the
  //  processing window in the scene. The values should be in the range of 0
  //  to 1, inclusive and in multiples of 1/1023.
  ni_rational_t bezier_curve_anchors[15];

  //  This flag shall be equal to 0 in bitstreams conforming to this version of
  //  this Specification. Other values are reserved for future use.
  uint8_t color_saturation_mapping_flag;

  //  The color saturation gain in the processing window in the scene. The
  //  value shall be in the range of 0 to 63/8, inclusive and in multiples of
  //  1/8. The default value shall be 1.
  ni_rational_t color_saturation_weight;
} ni_hdr_plus_color_transform_params_t;

// struct representing dynamic metadata for color volume transform -
// application 4 of SMPTE 2094-40:2016 standard.
typedef struct _ni_dynamic_hdr_plus
{
  // Country code by Rec. ITU-T T.35 Annex A. The value shall be 0xB5.
  uint8_t itu_t_t35_country_code;

  //  Application version in the application defining document in ST-2094
  //  suite. The value shall be set to 0.
  uint8_t application_version;

  //  The number of processing windows. The value shall be in the range
  //  of 1 to 3, inclusive.
  uint8_t num_windows;

  //  The color transform parameters for every processing window.
  ni_hdr_plus_color_transform_params_t params[3];

  //  The nominal maximum display luminance of the targeted system display,
  //  in units of 0.0001 candelas per square metre. The value shall be in
  //  the range of 0 to 10000, inclusive.
  ni_rational_t targeted_system_display_maximum_luminance;

  //  This flag shall be equal to 0 in bit streams conforming to this version
  //  of this Specification. The value 1 is reserved for future use.
  uint8_t targeted_system_display_actual_peak_luminance_flag;

  //  The number of rows in the targeted system_display_actual_peak_luminance
  //  array. The value shall be in the range of 2 to 25, inclusive.
  uint8_t num_rows_targeted_system_display_actual_peak_luminance;

  //  The number of columns in the
  //  targeted_system_display_actual_peak_luminance array. The value shall be
  //  in the range of 2 to 25, inclusive.
  uint8_t num_cols_targeted_system_display_actual_peak_luminance;

  //  The normalized actual peak luminance of the targeted system display. The
  //  values should be in the range of 0 to 1, inclusive and in multiples of
  //  1/15.
  ni_rational_t targeted_system_display_actual_peak_luminance[25][25];

  //  This flag shall be equal to 0 in bitstreams conforming to this version of
  //  this Specification. The value 1 is reserved for future use.
  uint8_t mastering_display_actual_peak_luminance_flag;

  //  The number of rows in the mastering_display_actual_peak_luminance array.
  //  The value shall be in the range of 2 to 25, inclusive.
  uint8_t num_rows_mastering_display_actual_peak_luminance;

  //  The number of columns in the mastering_display_actual_peak_luminance
  //  array. The value shall be in the range of 2 to 25, inclusive.
  uint8_t num_cols_mastering_display_actual_peak_luminance;

  //  The normalized actual peak luminance of the mastering display used for
  //  mastering the image essence. The values should be in the range of 0 to 1,
  //  inclusive and in multiples of 1/15.
  ni_rational_t mastering_display_actual_peak_luminance[25][25];
} ni_dynamic_hdr_plus_t;

#endif // #ifndef QUADRA

#define NI_LOGAN_NAL_VPS_BIT                  (0x01)
#define NI_LOGAN_NAL_SPS_BIT                  (0x01<<1)
#define NI_LOGAN_NAL_PPS_BIT                  (0x01<<2)
#define NI_LOGAN_GENERATE_ALL_NAL_HEADER_BIT  (0x01<<3)

/*
 * Table 7-1 – NAL unit type codes, syntax element categories, and NAL unit type classes in
 * T-REC-H.264-201704
 */
typedef enum _ni_logan_h264_nalu_type
{
  NI_LOGAN_H264_NAL_UNSPECIFIED     = 0,
  NI_LOGAN_H264_NAL_SLICE           = 1,
  NI_LOGAN_H264_NAL_DPA             = 2,
  NI_LOGAN_H264_NAL_DPB             = 3,
  NI_LOGAN_H264_NAL_DPC             = 4,
  NI_LOGAN_H264_NAL_IDR_SLICE       = 5,
  NI_LOGAN_H264_NAL_SEI             = 6,
  NI_LOGAN_H264_NAL_SPS             = 7,
  NI_LOGAN_H264_NAL_PPS             = 8,
  NI_LOGAN_H264_NAL_AUD             = 9,
  NI_LOGAN_H264_NAL_END_SEQUENCE    = 10,
  NI_LOGAN_H264_NAL_END_STREAM      = 11,
  NI_LOGAN_H264_NAL_FILLER_DATA     = 12,
  NI_LOGAN_H264_NAL_SPS_EXT         = 13,
  NI_LOGAN_H264_NAL_PREFIX          = 14,
  NI_LOGAN_H264_NAL_SUB_SPS         = 15,
  NI_LOGAN_H264_NAL_DPS             = 16,
  NI_LOGAN_H264_NAL_RESERVED17      = 17,
  NI_LOGAN_H264_NAL_RESERVED18      = 18,
  NI_LOGAN_H264_NAL_AUXILIARY_SLICE = 19,
  NI_LOGAN_H264_NAL_EXTEN_SLICE     = 20,
  NI_LOGAN_H264_NAL_DEPTH_EXTEN_SLICE = 21,
  NI_LOGAN_H264_NAL_RESERVED22      = 22,
  NI_LOGAN_H264_NAL_RESERVED23      = 23,
  NI_LOGAN_H264_NAL_UNSPECIFIED24   = 24,
  NI_LOGAN_H264_NAL_UNSPECIFIED25   = 25,
  NI_LOGAN_H264_NAL_UNSPECIFIED26   = 26,
  NI_LOGAN_H264_NAL_UNSPECIFIED27   = 27,
  NI_LOGAN_H264_NAL_UNSPECIFIED28   = 28,
  NI_LOGAN_H264_NAL_UNSPECIFIED29   = 29,
  NI_LOGAN_H264_NAL_UNSPECIFIED30   = 30,
  NI_LOGAN_H264_NAL_UNSPECIFIED31   = 31,
} ni_logan_h264_nalu_type_t;

/**
 * Table 7-1 – NAL unit type codes and NAL unit type classes in
 * T-REC-H.265-201802
 */
typedef enum _ni_logan_hevc_nalu_type
{
  NI_LOGAN_HEVC_NAL_TRAIL_N        = 0,
  NI_LOGAN_HEVC_NAL_TRAIL_R        = 1,
  NI_LOGAN_HEVC_NAL_TSA_N          = 2,
  NI_LOGAN_HEVC_NAL_TSA_R          = 3,
  NI_LOGAN_HEVC_NAL_STSA_N         = 4,
  NI_LOGAN_HEVC_NAL_STSA_R         = 5,
  NI_LOGAN_HEVC_NAL_RADL_N         = 6,
  NI_LOGAN_HEVC_NAL_RADL_R         = 7,
  NI_LOGAN_HEVC_NAL_RASL_N         = 8,
  NI_LOGAN_HEVC_NAL_RASL_R         = 9,
  NI_LOGAN_HEVC_NAL_VCL_N10        = 10,
  NI_LOGAN_HEVC_NAL_VCL_R11        = 11,
  NI_LOGAN_HEVC_NAL_VCL_N12        = 12,
  NI_LOGAN_HEVC_NAL_VCL_R13        = 13,
  NI_LOGAN_HEVC_NAL_VCL_N14        = 14,
  NI_LOGAN_HEVC_NAL_VCL_R15        = 15,
  NI_LOGAN_HEVC_NAL_BLA_W_LP       = 16,
  NI_LOGAN_HEVC_NAL_BLA_W_RADL     = 17,
  NI_LOGAN_HEVC_NAL_BLA_N_LP       = 18,
  NI_LOGAN_HEVC_NAL_IDR_W_RADL     = 19,
  NI_LOGAN_HEVC_NAL_IDR_N_LP       = 20,
  NI_LOGAN_HEVC_NAL_CRA_NUT        = 21,
  NI_LOGAN_HEVC_NAL_RSV_IRAP_VCL22 = 22,
  NI_LOGAN_HEVC_NAL_RSV_IRAP_VCL23 = 23,
  NI_LOGAN_HEVC_NAL_RSV_VCL24      = 24,
  NI_LOGAN_HEVC_NAL_RSV_VCL25      = 25,
  NI_LOGAN_HEVC_NAL_RSV_VCL26      = 26,
  NI_LOGAN_HEVC_NAL_RSV_VCL27      = 27,
  NI_LOGAN_HEVC_NAL_RSV_VCL28      = 28,
  NI_LOGAN_HEVC_NAL_RSV_VCL29      = 29,
  NI_LOGAN_HEVC_NAL_RSV_VCL30      = 30,
  NI_LOGAN_HEVC_NAL_RSV_VCL31      = 31,
  NI_LOGAN_HEVC_NAL_VPS            = 32,
  NI_LOGAN_HEVC_NAL_SPS            = 33,
  NI_LOGAN_HEVC_NAL_PPS            = 34,
  NI_LOGAN_HEVC_NAL_AUD            = 35,
  NI_LOGAN_HEVC_NAL_EOS_NUT        = 36,
  NI_LOGAN_HEVC_NAL_EOB_NUT        = 37,
  NI_LOGAN_HEVC_NAL_FD_NUT         = 38,
  NI_LOGAN_HEVC_NAL_SEI_PREFIX     = 39,
  NI_LOGAN_HEVC_NAL_SEI_SUFFIX     = 40,
  NI_LOGAN_HEVC_NAL_RSV_NVCL41     = 41,
  NI_LOGAN_HEVC_NAL_RSV_NVCL42     = 42,
  NI_LOGAN_HEVC_NAL_RSV_NVCL43     = 43,
  NI_LOGAN_HEVC_NAL_RSV_NVCL44     = 44,
  NI_LOGAN_HEVC_NAL_RSV_NVCL45     = 45,
  NI_LOGAN_HEVC_NAL_RSV_NVCL46     = 46,
  NI_LOGAN_HEVC_NAL_RSV_NVCL47     = 47,
  NI_LOGAN_HEVC_NAL_UNSPEC48       = 48,
  NI_LOGAN_HEVC_NAL_UNSPEC49       = 49,
  NI_LOGAN_HEVC_NAL_UNSPEC50       = 50,
  NI_LOGAN_HEVC_NAL_UNSPEC51       = 51,
  NI_LOGAN_HEVC_NAL_UNSPEC52       = 52,
  NI_LOGAN_HEVC_NAL_UNSPEC53       = 53,
  NI_LOGAN_HEVC_NAL_UNSPEC54       = 54,
  NI_LOGAN_HEVC_NAL_UNSPEC55       = 55,
  NI_LOGAN_HEVC_NAL_UNSPEC56       = 56,
  NI_LOGAN_HEVC_NAL_UNSPEC57       = 57,
  NI_LOGAN_HEVC_NAL_UNSPEC58       = 58,
  NI_LOGAN_HEVC_NAL_UNSPEC59       = 59,
  NI_LOGAN_HEVC_NAL_UNSPEC60       = 60,
  NI_LOGAN_HEVC_NAL_UNSPEC61       = 61,
  NI_LOGAN_HEVC_NAL_UNSPEC62       = 62,
  NI_LOGAN_HEVC_NAL_UNSPEC63       = 63,
} ni_logan_hevc_nalu_type_t;

/*!*****************************************************************************
 *  \brief  Whether SEI should be sent together with this frame to encoder
 *
 *  \param[in]  p_enc_ctx encoder session context
 *  \param[in]  pic_type frame type
 *  \param[in]  p_param encoder parameters
 *
 *  \return 1 if yes, 0 otherwise
 ******************************************************************************/
LIB_API int ni_logan_should_send_sei_with_frame(
    ni_logan_session_context_t* p_enc_ctx,
    ni_logan_pic_type_t pic_type,
    ni_logan_encoder_params_t *p_param);

/*!*****************************************************************************
 *  \brief  Set SPS VUI part of encoded stream header
 *
 *  \param[in/out]  p_param encoder parameters, its VUI data member will be
 *                  updated.
 *  \param[out]  p_ctx session context, its HRD parameters (hrd_params)
 *               may be updated.
 *  \param[in]  color_primaries color primaries
 *  \param[in]  color_trc color transfer characteristic
 *  \param[in]  color_space YUV colorspace type
 *  \param[in]  video_full_range_flag
 *  \param[in]  sar_num/sar_den sample aspect ration in numerator/denominator
 *  \param[in]  codec_format H.264 or H.265
 *
 *  \return NONE
 ******************************************************************************/
LIB_API void ni_logan_set_vui(ni_logan_encoder_params_t *p_param,
                              ni_logan_session_context_t *p_ctx,
                              ni_color_primaries_t color_primaries,
                              ni_color_transfer_characteristic_t color_trc,
                              ni_color_space_t color_space,
                              int video_full_range_flag,
                              int sar_num,
                              int sar_den,
                              ni_logan_codec_format_t codec_format);

/*!*****************************************************************************
 *  \brief  Buffered period sei
 *
 *  \param[in/out]  p_param encoder parameters, its GOP information will be
 *                  updated.
 *  \param[in]  p_ctx session context, use its HRD parameters (hrd_params).
 *  \param[in]  frame_idx frame index
 *  \param[in]  buf_len output buffer length
 *  \param[out]  p_buf output sei buffer.
 *
 *  \return put_bit_byte_size
 ******************************************************************************/
LIB_API uint32_t ni_logan_enc_buffering_period_sei(ni_logan_encoder_params_t *p_param,
                                                   ni_logan_session_context_t *p_ctx,
                                                   uint32_t frame_idx,
                                                   int buf_len,
                                                   uint8_t *p_buf);

/*!*****************************************************************************
 *  \brief  Buffered customer sei
 *
 *  \param[in]  p_data input side data of customer sei
 *  \param[out]  p_ctx session context, update its pkt_custom_sei.
 *  \param[in]  index fifo index
 *
 *  \return success return NI_LOGAN_RETCODE_SUCCESS
 *          failure return < 0
 ******************************************************************************/
LIB_API uint32_t ni_logan_enc_buffering_custom_sei(void *p_data,
                                                   ni_logan_session_context_t *p_ctx,
                                                   uint32_t index);

/*!*****************************************************************************
 *  \brief  Generate pic timing sei2
 *
 *  \param[in/out]  p_param encoder parameters, its GOP information will be
 *                  updated.
 *  \param[in/out]  p_ctx session context, its HRD parameters (hrd_params)
 *                  may be updated.
 *  \param[in]  is_i_or_idr True if it is i or idr packet.
 *  \param[in]  is_idr True if it is idr packet.
 *  \param[in]  frame_idx frame index
 *  \param[in]  buf_len output buffer length
 *  \param[out]  p_buf output sei buffer.
 *
 *  \return length of pci timing sei2
 ******************************************************************************/
LIB_API uint32_t ni_logan_enc_pic_timing_sei2(ni_logan_encoder_params_t *p_param,
                                              ni_logan_session_context_t *p_ctx,
                                              int is_i_or_idr,
                                              int is_idr,
                                              uint32_t frame_idx,
                                              int buf_len,
                                              uint8_t *p_buf);

/*!*****************************************************************************
 *  \brief  Retrieve auxiliary data (close caption, various SEI) associated with
 *          this frame that is returned by decoder, convert them to appropriate
 *          format and save them in the frame's auxiliary data storage for
 *          future use by encoding. Usually they would be sent together with
 *          this frame to encoder at encoding.
 *
 *  \param[in/out]  frame that is returned by decoder
 *
 *  \return NONE
 ******************************************************************************/
LIB_API void ni_logan_dec_retrieve_aux_data(ni_logan_frame_t *frame);

/*!*****************************************************************************
 *  \brief  Initialize auxiliary data that should be sent together with this frame
 *          to encoder based on the auxiliary data of the decoded frame.
 *
 *  \param[in/out]  ni_logan_session_data_io_t  session data
 *
 *  \return NONE
 ******************************************************************************/
LIB_API void ni_logan_enc_init_aux_params(ni_logan_session_data_io_t* p_api_fme);

/*!*****************************************************************************
 *  \brief  Prepare auxiliary data that should be sent together with this frame
 *          to encoder based on the auxiliary data of the decoded frame.
 *
 *  \param[in/out]  p_enc_ctx encoder session context whose various SEI type
 *                  header can be updated as the result of this function
 *  \param[out]  p_enc_frame frame to be sent to encoder
 *  \param[in]  p_dec_frame frame that is returned by decoder
 *  \param[in]  codec_format H.264 or H.265
 *  \param[in]  should_send_sei_with_frame if need to send a certain type of
 *              SEI with this frame
 *  \param[out]  mdcv_data SEI for HDR mastering display color volume info
 *  \param[out]  cll_data SEI for HDR content light level info
 *  \param[out]  cc_data SEI for close caption
 *  \param[out]  udu_data SEI for User data unregistered
 *  \param[out]  hdrp_data SEI for HDR10+
 *
 *  \return NONE
 ******************************************************************************/
LIB_API void ni_logan_enc_prep_aux_data(ni_logan_session_context_t* p_enc_ctx,
                                        ni_logan_frame_t *p_enc_frame,
                                        ni_logan_frame_t *p_dec_frame,
                                        ni_logan_codec_format_t codec_format,
                                        int should_send_sei_with_frame,
                                        uint8_t *mdcv_data,
                                        uint8_t *cll_data,
                                        uint8_t *cc_data,
                                        uint8_t *udu_data,
                                        uint8_t *hdrp_data);

/*!*****************************************************************************
 *  \brief  Copy auxiliary data that should be sent together with this frame
 *          to encoder.
 *
 *  \param[in]  p_enc_ctx encoder session context
 *  \param[out]  p_enc_frame frame to be sent to encoder
 *  \param[in]  p_dec_frame frame returned by decoder
 *  \param[in]  codec_format H.264 or H.265
 *  \param[in]  mdcv_data SEI for HDR mastering display color volume info
 *  \param[in]  cll_data SEI for HDR content light level info
 *  \param[in]  cc_data SEI for close caption
 *  \param[in]  udu_data SEI for User data unregistered
 *  \param[in]  hdrp_data SEI for HDR10+
 *
 *  \return NONE
 ******************************************************************************/
LIB_API void ni_logan_enc_copy_aux_data(ni_logan_session_context_t* p_enc_ctx,
                                        ni_logan_frame_t *p_enc_frame,
                                        ni_logan_frame_t *p_dec_frame,
                                        ni_logan_codec_format_t codec_format,
                                        const uint8_t *mdcv_data,
                                        const uint8_t *cll_data,
                                        const uint8_t *cc_data,
                                        const uint8_t *udu_data,
                                        const uint8_t *hdrp_data);

/*!*****************************************************************************
 *  \brief  Find the next start code
 *
 *  \param[in]  p pointer to buffer start address.
 *  \param[in]  end pointer to buffer end address.
 *  \param[state]  state pointer to nalu type address
 *
 *  \return search end address
 ******************************************************************************/
LIB_API const uint8_t *ni_logan_find_start_code(const uint8_t *p,
                                                const uint8_t *end,
                                                uint32_t *state);

/*!*****************************************************************************
 *  \brief  Fill reconfig params
 *
 *  \param[in]  p_param encoder parameters.
 *  \param[in/out]  p_ctx session context, its enc_change_params may be updated.
 *  \param[in/out]  p_api_fme extra_data_len and reconf_len may be updated
 *  \param[in]  reconfigCount reconfig count, may be updated.
 *
 *  \return reconfigCount on success
 *          < 0 on failure, NI_LOGAN_RETCODE_INVALID_PARAM
 *                          NI_LOGAN_RETCODE_ERROR_MEM_ALOC
 ******************************************************************************/
LIB_API int ni_logan_enc_fill_reconfig_params(ni_logan_encoder_params_t *p_param,
                                              ni_logan_session_context_t *p_ctx,
                                              ni_logan_frame_t *p_api_fme,
                                              int reconfigCount);

/*!******************************************************************************
 *  \brief  detect custom SEI payload data in Packet data,
 *          custom SEI has two meanings:
 *          a. the SEI type is not in the standard protocol, which is added by customes,
 *             for example SEI type 100, note that custom SEI is not user data unregistered SEI.
 *          b. the SEI NAL location does not conform to protocol. It's after VCL NALs.
 *          So there are cases to handle here:
 *          case a: enable custom_sei, detext custom SEIs before VCL.
 *          case b: enable custom_sei and enable_check_packet, detect custom SEIs before VCL,
 *                  and all SEIs after VCL.
 *          all of these SEIs are passthroughed in the same places after encoding.
 *
 *  \param uint8_t *data - packet data
 *  \param int size - pkt data size
 *  \param ni_logan_session_context_t *p_api_ctx - Netint session context
 *  \param ni_logan_decoder_params_t *api_param - Netint xcoder params
 *  \param ni_logan_packet_t *p_packet - netint internal packet
 *
 *  \return - 0 on success or not detect correct custom sei, non-0 on failure
 *******************************************************************************/
LIB_API int ni_logan_detect_custom_sei(const uint8_t *data,
                                       int size, 
                                       ni_logan_session_context_t *p_api_ctx,
                                       ni_logan_decoder_params_t *p_api_param,
                                       ni_logan_packet_t *p_packet);

/*!*****************************************************************************
 *  \brief  Fill reconfig vfr
 *
 *  \param[in/out]  p_session_ctx session context, some parameters may be updated.
 *  \param[in]  p_param encoder parameters.
 *  \param[in/out]  ni_frame aux_data may be updated
 *  \param[in]  pts frame pts
 *  \param[in]  timebase timebase
 *
 *  \return
 ******************************************************************************/
LIB_API void ni_logan_enc_reconfig_vfr(ni_logan_session_context_t *p_session_ctx,
                                       ni_logan_encoder_params_t *p_encoder_params,
                                       ni_logan_frame_t *ni_frame,
                                       int64_t pts,
                                       uint32_t timebase);

/*!*****************************************************************************
 *  \brief  Reset vui info from decoder
 *
 *  \param[in]  p_param encoder parameters.
 *  \param[in]  aux_data aux_data may be updated
 *  \param[in]  codec_format encoder format
 *
 *  \return
 ******************************************************************************/
LIB_API void ni_logan_reset_vui(ni_logan_encoder_params_t *p_param, ni_aux_data_t *aux_data,
                                ni_logan_codec_format_t codec_format);

/*!*****************************************************************************
 *  \brief  Fill reconfig vui
 *
 *  \param[in]  p_param encoder parameters.
 *  \param[in/out]  p_ctx session context, its enc_change_params may be updated.
 *  \param[in/out]  p_api_fme extra_data_len and reconf_len may be updated
 *  \param[in]  reconfigCount reconfig count, may be updated.
 *  \param[in]  aux_data aux_data may be updated
 *
 *  \return reconfigCount on success
 *          < 0 on failure, NI_LOGAN_RETCODE_INVALID_PARAM
 *                          NI_LOGAN_RETCODE_ERROR_MEM_ALOC
 ******************************************************************************/
LIB_API int ni_logan_enc_fill_reconfig_vui(ni_logan_encoder_params_t *p_param,
                                           ni_logan_session_context_t *p_ctx,
                                           ni_logan_frame_t *p_api_fme,
                                           int reconfigCount,
                                           ni_aux_data_t *aux_data);

#ifdef __cplusplus
}
#endif
