/*******************************************************************************
 *
 * Copyright (C) 2022 NETINT Technologies
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 ******************************************************************************/

/*!*****************************************************************************
*  \file   ni_enc_api_logan.c
*
*  \brief  NETINT encoder API file
*
*******************************************************************************/

#include "ni_enc_api_logan.h"

static uint8_t g_itu_t_t35_cc_sei_hdr_hevc[NI_CC_SEI_HDR_HEVC_LEN] =
{
  0x00, 0x00, 0x00, 0x01, // NAL start code 00 00 00 01
  0x4e,
  0x01, // nal_unit_header() {forbidden bit=0 nal_unit_type=39,
  // nuh_layer_id=0 nuh_temporal_id_plus1=1)
  0x04, // payloadType= 4 (user_data_registered_itu_t_t35)
  0 + 11, // payLoadSize= ui16Len + 11; to be set (index 7)
  0xb5, //  itu_t_t35_country_code =181 (North America)
  0x00,
  0x31, //  itu_t_t35_provider_code = 49
  0x47, 0x41, 0x39,
  0x34, // ATSC_user_identifier = "GA94"
  0x03, // ATSC1_data_user_data_type_code=3
  0 | 0xc0, // (ui16Len/3) | 0xc0 (to be set; index 16) (each CC character
  //is 3 bytes)
  0xFF  // em_data = 255
};

static uint8_t g_itu_t_t35_hdr10p_sei_hdr_hevc[NI_HDR10P_SEI_HDR_HEVC_LEN] =
{
  0x00, 0x00, 0x00, 0x01, // NAL start code 00 00 00 01
  0x4e,
  0x01, // nal_unit_header() {forbidden bit=0 nal_unit_type=39,
  // nuh_layer_id=0 nuh_temporal_id_plus1=1)
  0x04, // payloadType= 4 (user_data_registered_itu_t_t35)
  0x00, // payLoadSize; to be set (index 7)
  0xb5, //  u8 itu_t_t35_country_code =181 (North America)
  //0x00,
  //0x3c, //  u16 itu_t_t35_provider_code = 0x003c
  //0x00,
  //0x01, //  u16 itu_t_t35_provider_oriented_code = 0x0001
  // payLoadSize count starts from itu_t_t35_provider_code and goes until
  // and including trailer
};

static uint8_t g_itu_t_t35_cc_sei_hdr_h264[NI_CC_SEI_HDR_H264_LEN] =
{
  0x00, 0x00, 0x00, 0x01, // NAL start code 00 00 00 01
  0x06, // nal_unit_header() {forbidden bit=0 nal_ref_idc=0, nal_unit_type=6
  0x04, // payloadType= 4 (user_data_registered_itu_t_t35)
  0 + 11, // payLoadSize= ui16Len + 11; to be set (index 6)
  0xb5, //  itu_t_t35_country_code =181 (North America)
  0x00,
  0x31, //  itu_t_t35_provider_code = 49
  0x47, 0x41, 0x39,
  0x34, // ATSC_user_identifier = "GA94"
  0x03, // ATSC1_data_user_data_type_code=3
  0 | 0xc0, // (ui16Len/3) | 0xc0 (to be set; index 15) (each CC character
  //is 3 bytes)
  0xFF  // em_data = 255
};

static uint8_t g_itu_t_t35_hdr10p_sei_hdr_h264[NI_HDR10P_SEI_HDR_H264_LEN] =
{
  0x00, 0x00, 0x00, 0x01, // NAL start code 00 00 00 01
  0x06, // nal_unit_header() {forbidden bit=0 nal_ref_idc=0, nal_unit_type=6
  0x04, // payloadType= 4 (user_data_registered_itu_t_t35)
  0x00, // payLoadSize; to be set (index 6)
  0xb5, //  itu_t_t35_country_code =181 (North America)
  //0x00,
  //0x3c, //  u16 itu_t_t35_provider_code = 0x003c
  //0x00,
  //0x01, //  u16 itu_t_t35_provider_oriented_code = 0x0001
  // payLoadSize count starts from itu_t_t35_provider_code and goes until
  // and including trailer
};

static uint8_t g_sei_trailer[NI_CC_SEI_TRAILER_LEN] =
{
  0xFF, // marker_bits = 255
  0x80  // RBSP trailing bits - rbsp_stop_one_bit and 7 rbsp_alignment_zero_bit
};

/*!*****************************************************************************
 *  \brief  Initialize ni_logan_enc_context_t
 *
 *  \param[in/out] p_enc_ctx  Pointer to ni_logan_enc_context_t
 *
 *  \return
 ******************************************************************************/
static void ni_logan_enc_context_init(ni_logan_enc_context_t* p_enc_ctx)
{
  if (!p_enc_ctx)
  {
    ni_log(NI_LOG_ERROR, "%s(): line %d, Invalid Parameter\n",
           __FUNCTION__, __LINE__);
    return;
  }

  /* Variables that need to be initialized */
  p_enc_ctx->started = 0;
  p_enc_ctx->p_spsPpsHdr = NULL;
  p_enc_ctx->spsPpsAttach = 0;
  p_enc_ctx->spsPpsArrived = 0;
  p_enc_ctx->spsPpsHdrLen = 0;
  p_enc_ctx->firstPktArrived = 0;
  p_enc_ctx->dts_offset = 0;
  p_enc_ctx->reconfigCount = 0;
  p_enc_ctx->total_frames_received = 0;
  p_enc_ctx->first_frame_pts = 0;
  p_enc_ctx->latest_dts = 0;

  p_enc_ctx->orig_conf_win_top = 0;
  p_enc_ctx->orig_conf_win_bottom = 0;
  p_enc_ctx->orig_conf_win_left = 0;
  p_enc_ctx->orig_conf_win_left = 0;

  p_enc_ctx->extradata = NULL;
  p_enc_ctx->extradata_size = 0;

  p_enc_ctx->gotPacket = 0;
  p_enc_ctx->sentFrame = 0;

  p_enc_ctx->fps_number = 0;
  p_enc_ctx->fps_denominator = 0;

  p_enc_ctx->actual_dev_enc_idx = NI_INVALID_DEVICE_HANDLE;
  p_enc_ctx->actual_dev_name = NULL;

  p_enc_ctx->encoder_flushing = 0;
  p_enc_ctx->encoder_eof = 0;
  p_enc_ctx->eos_fme_received = 0;

  memset(&(p_enc_ctx->output_pkt), 0, sizeof(ni_logan_session_data_io_t));
  return;
}

/*!*****************************************************************************
 *  \brief  Initialize encode parameters, including ni_logan_enc_context_t,
 *          ni_logan_session_context_t and ni_logan_encoder_params_t
 *
 *  \param[in/out] p_enc_ctx  Pointer to ni_logan_enc_context_t
 *
 *  \return On success
 *                        NI_LOGAN_RETCODE_SUCCESS
 *          On failure
 *                        NI_LOGAN_RETCODE_FAILURE
 *                        NI_LOGAN_RETCODE_INVALID_PARAM,
 *                        NI_LOGAN_RETCODE_ERROR_MEM_ALOC, etc
 ******************************************************************************/
int ni_logan_encode_init(ni_logan_enc_context_t* p_enc_ctx)
{
  ni_logan_retcode_t retval = NI_LOGAN_RETCODE_SUCCESS;
  ni_logan_session_context_t *p_session_ctx = NULL;
  ni_logan_encoder_params_t *p_encoder_params = NULL;

  if (!p_enc_ctx)
  {
    ni_log(NI_LOG_ERROR, "%s(): line %d, Invalid Parameter\n",
           __FUNCTION__, __LINE__);
    return NI_LOGAN_RETCODE_INVALID_PARAM;
  }

  ni_log_set_level(ff_to_ni_log_level(p_enc_ctx->ff_log_level));

  ni_log(NI_LOG_TRACE, "%s() enter\n", __FUNCTION__);

  // resource management request
  if (0 == strcmp(p_enc_ctx->dev_xcoder, LIST_DEVICES_STR))
  {
    ni_log(NI_LOG_DEBUG, "XCoder: printing out all xcoder devices and "
           "their load, and exit ...\n");
    ni_logan_rsrc_print_all_devices_capability();
    return NI_LOGAN_RETCODE_FAILURE;
  }

  // Encoder Context init
  ni_logan_enc_context_init(p_enc_ctx);

  // Xcoder Session Context
  p_session_ctx = malloc(sizeof(ni_logan_session_context_t));
  if (!p_session_ctx)
  {
    ni_log(NI_LOG_ERROR, "ERROR %d:%s() line %d, alloc session context failed\n",
           NI_ERRNO, __FUNCTION__, __LINE__);
    retval = NI_LOGAN_RETCODE_ERROR_MEM_ALOC;
    LRETURN;
  }
  else
  {
    p_enc_ctx->p_session_ctx = p_session_ctx;
    ni_logan_device_session_context_init(p_session_ctx);

    /* this will be init again in ni_logan_encoder_session_open()
     * can be modified in the future */
    // init close caption SEI header and trailer
    memcpy(p_session_ctx->itu_t_t35_cc_sei_hdr_hevc, g_itu_t_t35_cc_sei_hdr_hevc,
           NI_CC_SEI_HDR_HEVC_LEN);
    memcpy(p_session_ctx->itu_t_t35_cc_sei_hdr_h264, g_itu_t_t35_cc_sei_hdr_h264,
           NI_CC_SEI_HDR_H264_LEN);
    memcpy(p_session_ctx->sei_trailer, g_sei_trailer, NI_CC_SEI_TRAILER_LEN);
    // init hdr10+ SEI header
    memcpy(p_session_ctx->itu_t_t35_hdr10p_sei_hdr_hevc, g_itu_t_t35_hdr10p_sei_hdr_hevc,
           NI_HDR10P_SEI_HDR_HEVC_LEN);
    memcpy(p_session_ctx->itu_t_t35_hdr10p_sei_hdr_h264, g_itu_t_t35_hdr10p_sei_hdr_h264,
           NI_HDR10P_SEI_HDR_H264_LEN);
  }

  // Xcoder User Configuration
  p_encoder_params = malloc(sizeof(ni_logan_encoder_params_t));
  if (!p_encoder_params)
  {
    ni_log(NI_LOG_ERROR, "ERROR %d:%s() line %d, alloc encoder params failed\n",
           NI_ERRNO, __FUNCTION__, __LINE__);
    retval = NI_LOGAN_RETCODE_ERROR_MEM_ALOC;
    LRETURN;
  }
  else
  {
    p_enc_ctx->p_encoder_params = p_encoder_params;
    retval = ni_logan_encoder_init_default_params(p_encoder_params,
                                                  p_enc_ctx->timebase_den,
                                                  p_enc_ctx->timebase_num * p_enc_ctx->ticks_per_frame,
                                                  p_enc_ctx->bit_rate,
                                                  NI_LOGAN_ODD2EVEN(p_enc_ctx->width),
                                                  NI_LOGAN_ODD2EVEN(p_enc_ctx->height));
    if (retval < 0)
    {
      switch (retval)
      {
        case NI_LOGAN_RETCODE_PARAM_ERROR_WIDTH_TOO_BIG:
          ni_log(NI_LOG_ERROR, "Invalid Picture Width: too big\n");
          break;
        case NI_LOGAN_RETCODE_PARAM_ERROR_WIDTH_TOO_SMALL:
          ni_log(NI_LOG_ERROR, "Invalid Picture Width: too small\n");
          break;
        case NI_LOGAN_RETCODE_PARAM_ERROR_HEIGHT_TOO_BIG:
          ni_log(NI_LOG_ERROR, "Invalid Picture Height: too big\n");
          break;
        case NI_LOGAN_RETCODE_PARAM_ERROR_HEIGHT_TOO_SMALL:
          ni_log(NI_LOG_ERROR, "Invalid Picture Height: too small\n");
          break;
        case NI_LOGAN_RETCODE_PARAM_ERROR_AREA_TOO_BIG:
          ni_log(NI_LOG_ERROR, "Invalid Picture Width x Height: exceeds %d\n",
                 NI_LOGAN_MAX_RESOLUTION_AREA);
          break;
        default:
            break;
      }
      ni_log(NI_LOG_ERROR, "%s(): line %d, %s\n",
             __FUNCTION__, __LINE__, ni_logan_err2str(retval));
      LRETURN;
    }
  }

  // Init input fifo
  p_enc_ctx->input_data_fifo = ni_logan_fifo_initialize(NI_LOGAN_INPUT_DATA_BUF_CNT_INIT,
                                                        sizeof(ni_logan_session_data_io_t));
  if (!p_enc_ctx->input_data_fifo)
  {
    ni_log(NI_LOG_ERROR, "%s(): line %d initialize enc frame buffer pool failed\n",
           __FUNCTION__, __LINE__);
    retval = NI_LOGAN_RETCODE_ERROR_MEM_ALOC;
    LRETURN;
  }

  END:

  if (retval != NI_LOGAN_RETCODE_SUCCESS)
  {
    free(p_enc_ctx->p_session_ctx);
    p_enc_ctx->p_session_ctx = NULL;
    free(p_encoder_params);
    p_enc_ctx->p_encoder_params = NULL;
    ni_logan_fifo_free(p_enc_ctx->input_data_fifo);
  }
  return retval;
}

/*!*****************************************************************************
 *  \brief  Parse encoder parameters
 *          Check whether the parameters are reasonable
 *
 *  \param[in/out] p_enc_ctx  Pointer to ni_logan_enc_context_t
 *
 *  \return On success
 *                        NI_LOGAN_RETCODE_SUCCESS
 *          On failure
 *                        NI_LOGAN_RETCODE_FAILURE,
 *                        NI_LOGAN_RETCODE_INVALID_PARAM, etc.
 ******************************************************************************/
int ni_logan_encode_params_parse(ni_logan_enc_context_t *p_enc_ctx)
{
  ni_logan_retcode_t retval = NI_LOGAN_RETCODE_SUCCESS;
  ni_logan_session_context_t *p_session_ctx = NULL;
  ni_logan_encoder_params_t *p_encoder_params = NULL;

  if (!p_enc_ctx)
  {
    ni_log(NI_LOG_ERROR, "%s(): line %d, Invalid Parameter\n",
           __FUNCTION__, __LINE__);
    return NI_LOGAN_RETCODE_INVALID_PARAM;
  }
  ni_log(NI_LOG_TRACE, "%s() enter\n", __FUNCTION__);

  p_session_ctx = p_enc_ctx->p_session_ctx;
  p_encoder_params = p_enc_ctx->p_encoder_params;
  if (!p_session_ctx || !p_encoder_params)
  {
    ni_log(NI_LOG_ERROR, "%s(): line %d, Invalid Parameter, %p, %p\n",
           __FUNCTION__, __LINE__, p_session_ctx, p_encoder_params);
    return NI_LOGAN_RETCODE_INVALID_PARAM;
  }

  // ni_session_context_t sync up ni_logan_enc_context_t
  p_session_ctx->codec_format = p_enc_ctx->codec_format;
  p_session_ctx->p_session_config = p_enc_ctx->p_encoder_params;
  p_session_ctx->hw_id = p_enc_ctx->dev_enc_idx;
  p_session_ctx->hw_name = p_enc_ctx->dev_enc_name;
  strcpy(p_session_ctx->dev_xcoder, p_enc_ctx->dev_xcoder);

  //overwrite keep alive timeout value here with a custom value if it was provided
  // provided
  // if xcoder option is set then overwrite the (legacy) decoder option
  uint32_t xcoder_timeout = p_encoder_params->enc_input_params.keep_alive_timeout;
  if (xcoder_timeout != NI_LOGAN_DEFAULT_KEEP_ALIVE_TIMEOUT)
  {
    p_session_ctx->keep_alive_timeout = xcoder_timeout;
  }
  else
  {
    p_session_ctx->keep_alive_timeout = p_enc_ctx->keep_alive_timeout;
  }

  //overwrite set_high_priority value here with a custom value if it was provided
  uint32_t xcoder_high_priority = p_encoder_params->enc_input_params.set_high_priority;
  if(xcoder_high_priority != 0)
  {
    p_session_ctx->set_high_priority = xcoder_high_priority;
  }
  else
  {
    p_session_ctx->set_high_priority = p_enc_ctx->set_high_priority;
  }

  // GOP PARSE
  switch (p_encoder_params->enc_input_params.gop_preset_index)
  {
    /* dts_offset is the max number of non-reference frames in a GOP
     * (derived from x264/5 algo) In case of IBBBP the first dts of the I frame should be input_pts-(3*ticks_per_frame)
     * In case of IBP the first dts of the I frame should be input_pts-(1*ticks_per_frame)
     * thus we ensure pts>dts in all cases
     * */
    case 1 /*PRESET_IDX_ALL_I*/:
    case 2 /*PRESET_IDX_IPP*/:
    case 6 /*PRESET_IDX_IPPPP*/:
    case 9 /*PRESET_IDX_SP*/:
      p_enc_ctx->dts_offset = 0;
      break;
    /* ts requires dts/pts of I frame not same when there are B frames in streams */
    case 3 /*PRESET_IDX_IBBB*/:
    case 4 /*PRESET_IDX_IBPBP*/:
    case 7 /*PRESET_IDX_IBBBB*/:
      p_enc_ctx->dts_offset = 1;
      break;
    case 5 /*PRESET_IDX_IBBBP*/:
      p_enc_ctx->dts_offset = 2;
        break;
    case 8 /*PRESET_IDX_RA_IB*/:
      p_enc_ctx->dts_offset = 3;
      break;
    default:
      // TBD need user to specify offset
      p_enc_ctx->dts_offset = 7;
      ni_log(NI_LOG_DEBUG, "dts offset default to 7, TBD\n");
      break;
  }
  if (1 == p_encoder_params->force_frame_type)
  {
    p_enc_ctx->dts_offset = 7;
  }

  p_session_ctx->src_bit_depth = 8;
  p_session_ctx->src_endian = NI_LOGAN_FRAME_LITTLE_ENDIAN;
  p_session_ctx->roi_len = 0;
  p_session_ctx->roi_avg_qp = 0;
  p_session_ctx->bit_depth_factor = 1;

  if (NI_LOGAN_PIX_FMT_YUV420P10BE == p_enc_ctx->pix_fmt ||
      NI_LOGAN_PIX_FMT_YUV420P10LE == p_enc_ctx->pix_fmt)
  {
    p_session_ctx->bit_depth_factor = 2;
    p_session_ctx->src_bit_depth = 10;
    if (NI_LOGAN_PIX_FMT_YUV420P10BE == p_enc_ctx->pix_fmt)
    {
      p_session_ctx->src_endian = NI_LOGAN_FRAME_BIG_ENDIAN;
    }
  }

  // init HDR SEI stuff
  p_session_ctx->sei_hdr_content_light_level_info_len =
  p_session_ctx->light_level_data_len =
  p_session_ctx->sei_hdr_mastering_display_color_vol_len =
  p_session_ctx->mdcv_max_min_lum_data_len = 0;
  p_session_ctx->p_master_display_meta_data = NULL;
  
  // init HRD SEI stuff (TBD: value after recovery ?)
  p_session_ctx->hrd_params.au_cpb_removal_delay_minus1 = 0;

  // DolbyVision, HRD and AUD settings
  if (NI_LOGAN_CODEC_FORMAT_H265 == p_enc_ctx->codec_format)
  {
    if (5 == p_encoder_params->dolby_vision_profile)
    {
      p_encoder_params->hrd_enable = 1;
      p_encoder_params->enable_aud = 1;
      p_encoder_params->enc_input_params.forced_header_enable = NI_LOGAN_ENC_REPEAT_HEADERS_ALL_KEY_FRAMES;
      p_encoder_params->enc_input_params.decoding_refresh_type = 2;
    }
    if (p_encoder_params->hrd_enable)
    {
      p_encoder_params->enc_input_params.rc.enable_rate_control = 1;
    }
  }

  if (ni_logan_encoder_params_check(p_encoder_params, p_enc_ctx->codec_format) !=
      NI_LOGAN_RETCODE_SUCCESS)
  {
    ni_log(NI_LOG_ERROR, "Validate encode parameters failed\n");
    return NI_LOGAN_RETCODE_INVALID_PARAM;
  }

  /* Update parameters that may be changed through xcoder parameters */
  p_enc_ctx->fps_number = p_encoder_params->fps_number;
  p_enc_ctx->fps_denominator = p_encoder_params->fps_denominator;
  p_enc_ctx->bit_rate = p_encoder_params->bitrate;

  if (p_encoder_params->enable_vfr)
  {
    //in the vfr mode, Customer WangSu may reset time base to a very large value, such as 1000.
    //At this time, the calculated framerate depends on timebase and ticket_per_frame is incorrect.
    //So we choose to set the default framerate 30.
    //If the calucluated framerate is correct, we will keep the original calculated framerate value
    //Assume the frame between 5-120 is correct.
    //using the time_base to initial timing info
    if (p_encoder_params->enc_input_params.frame_rate < 5 ||
        p_encoder_params->enc_input_params.frame_rate > 120)
    {
      p_encoder_params->enc_input_params.frame_rate = 30;
    }
    p_session_ctx->ui32timing_scale = p_enc_ctx->timebase_den;
    p_session_ctx->ui32num_unit_in_tick = p_enc_ctx->timebase_num;
    p_session_ctx->prev_bitrate = p_enc_ctx->bit_rate;
    p_session_ctx->init_bitrate = p_enc_ctx->bit_rate;
    p_session_ctx->last_change_framenum = 0;
    p_session_ctx->fps_change_detect_count = 0;
  }

  // generate encoded bitstream headers in advance if configured to do so
  if (p_encoder_params->generate_enc_hdrs)
  {
    retval = ni_logan_encode_header(p_enc_ctx);
  }

  ni_log(NI_LOG_TRACE, "XCoder Encoder Session Context:\n");

  ni_log(NI_LOG_TRACE, "hw_id=%d\n", p_session_ctx->hw_id);
  ni_log(NI_LOG_TRACE, "hw_name=%s\n", p_session_ctx->hw_name);
  ni_log(NI_LOG_TRACE, "codec_format=%u\n", p_session_ctx->codec_format);
  ni_log(NI_LOG_TRACE, "src_bit_depth=%d\n", p_session_ctx->src_bit_depth);
  ni_log(NI_LOG_TRACE, "src_endian=%d\n", p_session_ctx->src_endian);
  ni_log(NI_LOG_TRACE, "bit_depth_factor=%d\n", p_session_ctx->bit_depth_factor);
  ni_log(NI_LOG_TRACE, "keep_alive_timeout=%u\n", p_session_ctx->keep_alive_timeout);
  ni_log(NI_LOG_TRACE, "set_high_priority=%u\n", p_session_ctx->set_high_priority);

  ni_log(NI_LOG_TRACE, "timebase_num=%d\n", p_enc_ctx->timebase_num);
  ni_log(NI_LOG_TRACE, "timebase_den=%d\n", p_enc_ctx->timebase_den);
  ni_log(NI_LOG_TRACE, "ticks_per_frame=%d\n", p_enc_ctx->ticks_per_frame);
  ni_log(NI_LOG_TRACE, "bit_rate=%d\n", p_enc_ctx->bit_rate);
  ni_log(NI_LOG_TRACE, "width=%d\n", p_enc_ctx->width);
  ni_log(NI_LOG_TRACE, "height=%d\n", p_enc_ctx->height);
  ni_log(NI_LOG_TRACE, "pix_fmt=%d\n", p_enc_ctx->pix_fmt);

  ni_log(NI_LOG_TRACE, "color_primaries=%d\n", p_enc_ctx->color_primaries);
  ni_log(NI_LOG_TRACE, "color_trc=%d\n", p_enc_ctx->color_trc);
  ni_log(NI_LOG_TRACE, "color_space=%d\n", p_enc_ctx->color_space);
  ni_log(NI_LOG_TRACE, "color_range=%d\n", p_enc_ctx->color_range);

  ni_log(NI_LOG_TRACE, "sar_num=%d\n", p_enc_ctx->sar_num);
  ni_log(NI_LOG_TRACE, "sar_den=%d\n", p_enc_ctx->sar_den);

  return retval;
}

/*!*****************************************************************************
 *  \brief  Get encoder headers from the hardware
 *
 *  \param[in/out] p_enc_ctx  Pointer to ni_logan_enc_context_t
 *
 *  \return On success
 *                        NI_LOGAN_RETCODE_SUCCESS
 *          On failure
 *                        NI_LOGAN_RETCODE_FAILURE,
 *                        NI_LOGAN_RETCODE_ERROR_MEM_ALOC, etc
 ******************************************************************************/
int ni_logan_encode_header(ni_logan_enc_context_t *p_enc_ctx)
{
  int ret = NI_LOGAN_RETCODE_SUCCESS;
  int recv = 0;
  ni_logan_session_context_t session_ctx = {0};
  ni_logan_encoder_params_t *p_encoder_param = p_enc_ctx->p_encoder_params;
  ni_logan_session_data_io_t session_data_pkt = {0};

  ni_log(NI_LOG_TRACE, "%s() enter\n", __FUNCTION__);

  // use a copy of encoder context, take care to restore original config
  // cropping setting
  memcpy(&session_ctx, p_enc_ctx->p_session_ctx, sizeof(ni_logan_session_context_t));

  int orig_conf_win_right = p_encoder_param->enc_input_params.conf_win_right;
  int orig_conf_win_bottom = p_encoder_param->enc_input_params.conf_win_bottom;
  ni_logan_session_context_t *p_orig_session_ctx = p_enc_ctx->p_session_ctx;
  p_enc_ctx->p_session_ctx = &session_ctx;

  // set to null when to generate encoder header
  p_enc_ctx->p_input_fme = NULL;
  ret = ni_logan_encode_open(p_enc_ctx);
  if (ret != NI_LOGAN_RETCODE_SUCCESS)
  {
    ni_log(NI_LOG_ERROR, "ERROR %d:%s() line %d, failed to open encoder, ret=%d\n",
           __FUNCTION__, __LINE__, ret);
    return ret;
  }

  ni_logan_packet_t *p_ni_pkt = &session_data_pkt.data.packet;
  ni_logan_packet_buffer_alloc(p_ni_pkt, NI_LOGAN_MAX_TX_SZ);

  for (; ;)
  {
    recv = ni_logan_device_session_read(&session_ctx, &session_data_pkt,
                                        NI_LOGAN_DEVICE_TYPE_ENCODER);

    if (recv > 0)
    {
      p_enc_ctx->extradata_size = recv - NI_LOGAN_FW_ENC_BITSTREAM_META_DATA_SIZE;
      p_enc_ctx->extradata = malloc(p_enc_ctx->extradata_size);
      if (!p_enc_ctx->extradata)
      {
        ni_log(NI_LOG_ERROR, "ERROR %d:%s() line %d, alloc extradata failed\n",
               NI_ERRNO, __FUNCTION__, __LINE__);
        p_enc_ctx->extradata_size = 0;
        recv = NI_LOGAN_RETCODE_ERROR_MEM_ALOC;
      }
      else
      {
        memset(p_enc_ctx->extradata, 0, p_enc_ctx->extradata_size);
        memcpy(p_enc_ctx->extradata,
               p_ni_pkt->p_data + NI_LOGAN_FW_ENC_BITSTREAM_META_DATA_SIZE,
               p_enc_ctx->extradata_size);
      }

      ni_log(NI_LOG_TRACE, "%s extradata_size: %d\n", __FUNCTION__,
             p_enc_ctx->extradata_size);
      break;
    }
    else if (recv == NI_LOGAN_RETCODE_SUCCESS)
    {
      continue;
    }
    else
    {
      ni_log(NI_LOG_ERROR, "%s session read error: %d", __FUNCTION__, recv);
      break;
    }
  }

  // close and clean up the temporary session
  ret = ni_logan_device_session_close(&session_ctx, 0, NI_LOGAN_DEVICE_TYPE_ENCODER);

#ifdef _WIN32
  ni_logan_device_close(session_ctx.device_handle);
#elif __linux__
  ni_logan_device_close(session_ctx.device_handle);
  ni_logan_device_close(session_ctx.blk_io_handle);
#endif
  session_ctx.device_handle = NI_INVALID_DEVICE_HANDLE;
  session_ctx.blk_io_handle = NI_INVALID_DEVICE_HANDLE;

  ni_logan_packet_buffer_free(p_ni_pkt);

  p_encoder_param->enc_input_params.conf_win_right = orig_conf_win_right;
  p_encoder_param->enc_input_params.conf_win_bottom = orig_conf_win_bottom;
  p_enc_ctx->actual_dev_enc_idx = NI_INVALID_DEVICE_HANDLE;
  p_enc_ctx->actual_dev_name = NULL;
  p_enc_ctx->p_session_ctx = p_orig_session_ctx;

  return (recv < 0 ? recv : ret);

}

/*!*****************************************************************************
 *  \brief  Open encoder
 *
 *  \param[in/out] p_enc_ctx  Pointer to ni_logan_enc_context_t
 *
 *  \return On success
 *                        NI_LOGAN_RETCODE_SUCCESS
 *          On failure
 *                        NI_LOGAN_RETCODE_FAILURE
 *                        NI_LOGAN_RETCODE_INVALID_PARAM, etc.
 ******************************************************************************/
int ni_logan_encode_open(ni_logan_enc_context_t *p_enc_ctx)
{
  int ret = 0;
  ni_logan_session_context_t *p_session_ctx = NULL;
  ni_logan_encoder_params_t *p_encoder_param = NULL;
  ni_logan_frame_t *p_in_frame = NULL;
  int frame_width;
  int frame_height;
  int linesize_aligned = 0;
  int height_aligned = 0;
  int video_full_range_flag = 0;
  // color metrics
  ni_color_primaries_t color_primaries;
  ni_color_transfer_characteristic_t color_trc;
  ni_color_space_t color_space;

  if (!p_enc_ctx || !p_enc_ctx->p_session_ctx || !p_enc_ctx->p_encoder_params)
  {
    ni_log(NI_LOG_ERROR, "%s(): line %d, Invalid Parameter, %p, %p, %p\n",
           __FUNCTION__, __LINE__, p_enc_ctx, p_enc_ctx->p_session_ctx,
           p_enc_ctx->p_encoder_params);
    return NI_LOGAN_RETCODE_INVALID_PARAM;
  }

  ni_log(NI_LOG_TRACE, "%s() enter\n", __FUNCTION__);

  p_session_ctx = p_enc_ctx->p_session_ctx;
  p_encoder_param = p_enc_ctx->p_encoder_params;
  p_in_frame = p_enc_ctx->p_input_fme;
  color_primaries = p_enc_ctx->color_primaries;
  color_trc = p_enc_ctx->color_trc;
  color_space = p_enc_ctx->color_space;

  if (p_in_frame && p_in_frame->video_orig_width > 0 && p_in_frame->video_orig_height > 0)
  {
    frame_width = NI_LOGAN_ODD2EVEN(p_in_frame->video_orig_width);
    frame_height = NI_LOGAN_ODD2EVEN(p_in_frame->video_orig_height);
    color_primaries = p_in_frame->color_primaries;
    color_trc = p_in_frame->color_trc;
    color_space = p_in_frame->color_space;
    // Force frame color metrics if specified in command line
    if (p_in_frame->color_primaries != p_enc_ctx->color_primaries &&
        p_enc_ctx->color_primaries != NI_COL_PRI_UNSPECIFIED)
    {
      color_primaries = p_enc_ctx->color_primaries;
    }
    if (p_in_frame->color_trc != p_enc_ctx->color_trc &&
        p_enc_ctx->color_trc != NI_COL_TRC_UNSPECIFIED)
    {
      color_trc = p_enc_ctx->color_trc;
    }
    if (p_in_frame->color_space != p_enc_ctx->color_space &&
        p_enc_ctx->color_space != NI_COL_SPC_UNSPECIFIED)
    {
      color_space = p_enc_ctx->color_space;
    }
  }
  else
  {
    frame_width = NI_LOGAN_ODD2EVEN(p_enc_ctx->width);
    frame_height = NI_LOGAN_ODD2EVEN(p_enc_ctx->height);
    color_primaries = p_enc_ctx->color_primaries;
    color_trc = p_enc_ctx->color_trc;
    color_space = p_enc_ctx->color_space;
  }

  linesize_aligned = ((frame_width + 7) / 8) * 8;
  if (p_enc_ctx->codec_format == NI_LOGAN_CODEC_FORMAT_H264)
  {
    linesize_aligned = ((frame_width + 15) / 16) * 16;
  }

  // save the original ConformanceWindowOffsets before changed
  p_enc_ctx->orig_conf_win_right = p_encoder_param->enc_input_params.conf_win_right;
  p_enc_ctx->orig_conf_win_bottom = p_encoder_param->enc_input_params.conf_win_bottom;
  if (linesize_aligned < NI_LOGAN_MIN_WIDTH)
  {
    p_encoder_param->enc_input_params.conf_win_right += NI_LOGAN_MIN_WIDTH - frame_width;
    linesize_aligned = NI_LOGAN_MIN_WIDTH;
  }
  else if (linesize_aligned > frame_width)
  {
    p_encoder_param->enc_input_params.conf_win_right += linesize_aligned - frame_width;
  }
  p_encoder_param->source_width = linesize_aligned;

  height_aligned = ((frame_height + 7) / 8) * 8;
  if (p_enc_ctx->codec_format== NI_LOGAN_CODEC_FORMAT_H264) {
    height_aligned = ((frame_height + 15) / 16) * 16;
  }

  if (height_aligned < NI_LOGAN_MIN_HEIGHT)
  {
    p_encoder_param->enc_input_params.conf_win_bottom += NI_LOGAN_MIN_HEIGHT - frame_height;
    p_encoder_param->source_height = NI_LOGAN_MIN_HEIGHT;
    height_aligned = NI_LOGAN_MIN_HEIGHT;
  }
  else if (height_aligned > frame_height)
  {
    p_encoder_param->enc_input_params.conf_win_bottom += height_aligned - frame_height;
    p_encoder_param->source_height = height_aligned;
  }

  // DolbyVision support
  if (5 == p_encoder_param->dolby_vision_profile &&
      NI_LOGAN_CODEC_FORMAT_H265 == p_enc_ctx->codec_format)
  {
    color_primaries = color_trc = color_space = 2;
    video_full_range_flag = 1;
  }

  // According to the pixel format or color range from the incoming video
  if (p_enc_ctx->color_range == NI_COL_RANGE_JPEG ||
      p_enc_ctx->pix_fmt == NI_LOGAN_PIX_FMT_YUVJ420P)
  {
    ni_log(NI_LOG_TRACE, "%s set video_full_range_flag\n", __FUNCTION__);
    video_full_range_flag = 1;
  }

  if (p_encoder_param->enable_vui_info_passthru == 1)
  {
    // The vui info should flow decoder, so skip ni_logan_set_vui
    ;
  }
  else if ((5 == p_encoder_param->dolby_vision_profile &&
       p_enc_ctx->codec_format== NI_LOGAN_CODEC_FORMAT_H265) ||
      color_primaries == NI_COL_PRI_BT2020 ||
      color_trc == NI_COL_TRC_SMPTE2084 ||
      color_trc == NI_COL_TRC_ARIB_STD_B67 ||
      color_space == NI_COL_SPC_BT2020_NCL ||
      color_space == NI_COL_SPC_BT2020_CL)
  {
    p_encoder_param->hdrEnableVUI = 1;
    ni_logan_set_vui(p_encoder_param,p_session_ctx,
                     color_primaries, color_trc, color_space,
                     video_full_range_flag, p_enc_ctx->sar_num,
                     p_enc_ctx->sar_den, p_enc_ctx->codec_format);
    ni_log(NI_LOG_TRACE, "XCoder HDR color info color_primaries: %d "
           "color_trc: %d  color_space %d sar %d/%d\n",
           color_primaries, color_trc, color_space,
           p_enc_ctx->sar_num, p_enc_ctx->sar_den);
  }
  else
  {
    p_encoder_param->hdrEnableVUI = 0;
    ni_logan_set_vui(p_encoder_param, p_session_ctx,
                     color_primaries, color_trc, color_space,
                     video_full_range_flag, p_enc_ctx->sar_num,
                     p_enc_ctx->sar_den, p_enc_ctx->codec_format);
  }

  ret = ni_logan_device_session_open(p_session_ctx, NI_LOGAN_DEVICE_TYPE_ENCODER);

  p_enc_ctx->actual_dev_name = p_session_ctx->dev_xcoder_name;
  p_enc_ctx->actual_dev_enc_idx = p_session_ctx->hw_id;

  if (ret != NI_LOGAN_RETCODE_SUCCESS)
  {
    ni_log(NI_LOG_ERROR, "%s(): error line %d, failed to open encoder, ret=%d\n",
           __FUNCTION__, __LINE__, ret);
    return ret;
  }
  else
  {
    ni_log(NI_LOG_TRACE, "%s(): encoder %s Index %d (inst: %d) opened successfully\n",
           __FUNCTION__, p_enc_ctx->actual_dev_name, p_enc_ctx->actual_dev_enc_idx,
           p_session_ctx->session_id);
  }
  return ret;
}

/*!*****************************************************************************
 *  \brief  Close encoder
 *
 *  \param[in/out] p_enc_ctx  Pointer to ni_logan_enc_context_t
 *
 *  \return On success
 *                        NI_LOGAN_RETCODE_SUCCESS
 *          On failure
 *                        NI_LOGAN_RETCODE_FAILURE
 *                        NI_LOGAN_RETCODE_INVALID_PARAM,
 *                        NI_LOGAN_RETCODE_ERROR_MEM_ALOC,
 *                        NI_LOGAN_RETCODE_ERROR_NVME_CMD_FAILED, etc
 ******************************************************************************/
int ni_logan_encode_close(ni_logan_enc_context_t *p_enc_ctx)
{
  int ret = NI_LOGAN_RETCODE_SUCCESS;
  ni_logan_session_context_t *p_session_ctx = NULL;
  ni_logan_session_data_io_t *p_ni_frame = NULL;

  if (!p_enc_ctx || !p_enc_ctx->p_session_ctx)
  {
    ni_log(NI_LOG_ERROR, "%s(): line %d, Invalid Parameter, %p, %p\n",
           __FUNCTION__, __LINE__, p_enc_ctx, p_enc_ctx->p_session_ctx);
    return NI_LOGAN_RETCODE_INVALID_PARAM;
  }
  ni_log(NI_LOG_TRACE, "%s() enter\n", __FUNCTION__);

  p_session_ctx = p_enc_ctx->p_session_ctx;
  ret = ni_logan_device_session_close(p_session_ctx,
                                      p_enc_ctx->encoder_eof,
                                      NI_LOGAN_DEVICE_TYPE_ENCODER);
  if (NI_LOGAN_RETCODE_SUCCESS != ret)
  {
    ni_log(NI_LOG_ERROR, "%s(): error line %d, failed to close encoder, ret=%d\n",
           __FUNCTION__, __LINE__, ret);
  }

#ifdef _WIN32
  ni_logan_device_close(p_session_ctx->device_handle);
#elif __linux__
  ni_logan_device_close(p_session_ctx->device_handle);
  ni_logan_device_close(p_session_ctx->blk_io_handle);
#endif
  p_session_ctx->device_handle = NI_INVALID_DEVICE_HANDLE;
  p_session_ctx->blk_io_handle = NI_INVALID_DEVICE_HANDLE;

  ni_logan_packet_buffer_free((&p_enc_ctx->output_pkt.data.packet));

  if (p_session_ctx->session_run_state != LOGAN_SESSION_RUN_STATE_RESETTING &&
      p_session_ctx->session_run_state != LOGAN_SESSION_RUN_STATE_SEQ_CHANGE_DRAINING)
  {
    if (p_enc_ctx->input_data_fifo)
    {
      // push back
      do{
        p_ni_frame = ni_logan_fifo_generic_write(p_enc_ctx->input_data_fifo);
      }while(p_ni_frame);
      // free all
      do{
        p_ni_frame = ni_logan_fifo_generic_read(p_enc_ctx->input_data_fifo);
        if (p_ni_frame)
        {
          ni_logan_frame_buffer_free(&(p_ni_frame->data.frame));
        }
      }while(p_ni_frame);
    }
    ni_logan_fifo_free(p_enc_ctx->input_data_fifo);
    ni_log(NI_LOG_TRACE, "%s(): encoder frame fifo freed.\n", __FUNCTION__);

    ni_logan_device_session_context_clear(p_enc_ctx->p_session_ctx);
    free(p_enc_ctx->p_session_ctx);
    p_enc_ctx->p_session_ctx = NULL;

    free(p_enc_ctx->p_encoder_params);
    p_enc_ctx->p_encoder_params = NULL;
  }
  else
  {
    ni_logan_device_session_context_clear(p_enc_ctx->p_session_ctx);
    ni_log(NI_LOG_TRACE, "%s(): encoder reset flow or sequence change flow.\n", __FUNCTION__);
  }

  free(p_enc_ctx->extradata);
  p_enc_ctx->extradata = NULL;
  p_enc_ctx->extradata_size = 0;

  free(p_enc_ctx->p_spsPpsHdr);
  p_enc_ctx->p_spsPpsHdr = NULL;
  p_enc_ctx->spsPpsHdrLen = 0;
  
  return ret;
}

/*!*****************************************************************************
 *  \brief  Reinit encoder in some situation
 *
 *  \param[in/out] p_enc_ctx  Pointer to ni_logan_enc_context_t
 *
 *  \return On success
 *                        NI_LOGAN_RETCODE_SUCCESS
 *          On failure
 *                        NI_LOGAN_RETCODE_FAILURE,
 *                        NI_LOGAN_RETCODE_INVALID_PARAM, etc.
 ******************************************************************************/
static int ni_logan_encode_reinit(ni_logan_enc_context_t *p_enc_ctx)
{
  int ret = NI_LOGAN_RETCODE_SUCCESS;
  ni_logan_session_context_t *p_session_ctx = p_enc_ctx->p_session_ctx;
  ni_logan_session_data_io_t *p_input_fme = NULL;
  ni_logan_encoder_params_t* p_encoder_params = p_enc_ctx->p_encoder_params;
  int orig_conf_win_bottom = p_enc_ctx->orig_conf_win_bottom;
  int orig_conf_win_right = p_enc_ctx->orig_conf_win_right;  
  ni_logan_session_run_state_t orig_run_state = p_session_ctx->session_run_state;

  ni_log(NI_LOG_TRACE, "%s() enter\n", __FUNCTION__);

  ni_logan_encode_close(p_enc_ctx);

  ni_logan_enc_context_init(p_enc_ctx);

  ni_logan_device_session_context_init(p_session_ctx);

  // for sequence change, re-init avctx's resolution to the changed one
  // that is stored in the first frame of the fifo
  if (orig_run_state == LOGAN_SESSION_RUN_STATE_SEQ_CHANGE_DRAINING)
  {
    p_input_fme = ni_logan_fifo_generic_read(p_enc_ctx->input_data_fifo);
    if (!p_input_fme)
    {
      ni_log(NI_LOG_ERROR, "%s(): fifo is empty when reinit\n", __FUNCTION__);
      ret = NI_LOGAN_RETCODE_FAILURE;
      return ret;
    }
    ni_log(NI_LOG_TRACE, "%s(): resolution changing %dx%d -> %dx%d\n",
           __FUNCTION__, p_enc_ctx->width, p_enc_ctx->height,
           p_input_fme->data.frame.video_width,
           p_input_fme->data.frame.video_height);
    p_enc_ctx->width = p_input_fme->data.frame.video_orig_width;
    p_enc_ctx->height = p_input_fme->data.frame.video_orig_height;
    ret = ni_logan_fifo_return_read(p_enc_ctx->input_data_fifo);
    if (ret < 0)
    {
      ni_log(NI_LOG_INFO, "%s(): Drop 1 frame because of fifo full, "
             "when encode reinit\n", __FUNCTION__);
    }

    // refill the encoder params
    p_encoder_params->source_width = NI_LOGAN_ODD2EVEN(p_enc_ctx->width);
    p_encoder_params->source_height = NI_LOGAN_ODD2EVEN(p_enc_ctx->height);
    if (p_encoder_params->source_width > NI_LOGAN_PARAM_MAX_WIDTH)
    {
      ret = NI_LOGAN_RETCODE_PARAM_ERROR_WIDTH_TOO_BIG;
      return ret;
    }
    if (p_encoder_params->source_width < NI_LOGAN_PARAM_MIN_WIDTH)
    {
      ret = NI_LOGAN_RETCODE_PARAM_ERROR_WIDTH_TOO_SMALL;
      return ret;
    }
    if (p_encoder_params->source_height > NI_LOGAN_PARAM_MAX_HEIGHT)
    {
      ret = NI_LOGAN_RETCODE_PARAM_ERROR_HEIGHT_TOO_BIG;
      return ret;
    }
    if (p_encoder_params->source_height < NI_LOGAN_PARAM_MIN_HEIGHT)
    {
      ret =  NI_LOGAN_RETCODE_PARAM_ERROR_HEIGHT_TOO_SMALL;
      return ret;
    }
    if (p_encoder_params->source_height*p_encoder_params->source_width > NI_LOGAN_MAX_RESOLUTION_AREA)
    {
      ret = NI_LOGAN_RETCODE_PARAM_ERROR_AREA_TOO_BIG;
      return ret;
    }
  }

  // these may be updated in ni_logan_encode_open()
  p_encoder_params->enc_input_params.conf_win_bottom = orig_conf_win_bottom;
  p_encoder_params->enc_input_params.conf_win_right = orig_conf_win_right;

  ret = ni_logan_encode_params_parse(p_enc_ctx);
  if (ret < 0)
  {
    ni_log(NI_LOG_ERROR, "%s(): line %d, %s\n",
           __FUNCTION__, __LINE__, ni_logan_err2str(ret));
    ret = NI_LOGAN_RETCODE_FAILURE;
    return ret;
  }

  while (!ni_logan_fifo_is_empty(p_enc_ctx->input_data_fifo))
  {
    p_session_ctx->session_run_state = LOGAN_SESSION_RUN_STATE_QUEUED_FRAME_DRAINING;
    ret = ni_logan_encode_send(p_enc_ctx);
    if (ret < 0)
    {
      ni_log(NI_LOG_ERROR, "%s(): line %d, encode send failed, %s\n",
             __FUNCTION__, __LINE__, ni_logan_err2str(ret));
      break;
    }
    else if ( p_session_ctx->session_run_state == LOGAN_SESSION_RUN_STATE_SEQ_CHANGE_DRAINING)
    {
      ni_log(NI_LOG_INFO, "%s(): break flush queued frames because of "
             "resolution changes again, left cnt=%d\n",
             __FUNCTION__, p_enc_ctx->input_data_fifo->number_of_buffers_used);
      break;
    }
    else if (p_session_ctx->status == NI_LOGAN_RETCODE_NVME_SC_WRITE_BUFFER_FULL)
    {
      ni_log(NI_LOG_INFO, "%s(): break flush queued frames because of "
             "buffer full, left cnt=%d\n",
             __FUNCTION__, p_enc_ctx->input_data_fifo->number_of_buffers_used);
      break;
    }
    else
    {
      ni_log(NI_LOG_TRACE, "%s(): continue to flush queued frames, cnt=%d\n",
             __FUNCTION__, p_enc_ctx->input_data_fifo->number_of_buffers_used);
    }
  }

  return ret;
}

/*!*****************************************************************************
 *  \brief  Get allocated ni_frame from fifo pool
 *
 *  \param[in]  p_enc_ctx Pointer to ni_logan_enc_context_t
 *
 *  \return On success
 *                        NI_LOGAN_RETCODE_SUCCESS
 *          On failure
 *                        NI_LOGAN_RETCODE_FAILURE
 ******************************************************************************/
int ni_logan_encode_get_frame(ni_logan_enc_context_t * p_enc_ctx)
{
  int ret = NI_LOGAN_RETCODE_SUCCESS;
  ni_logan_session_context_t *p_session_ctx = NULL;
  ni_logan_encoder_params_t *p_encoder_params = NULL;

  if (!p_enc_ctx || !p_enc_ctx->p_session_ctx || !p_enc_ctx->p_encoder_params)
  {
    ni_log(NI_LOG_ERROR, "%s(): line %d, Invalid Parameter, %p, %p, %p\n",
           __FUNCTION__, __LINE__, p_enc_ctx, p_enc_ctx->p_session_ctx,
           p_enc_ctx->p_encoder_params);
    return NI_LOGAN_RETCODE_INVALID_PARAM;
  }
  ni_log(NI_LOG_TRACE, "%s() enter\n", __FUNCTION__);

  p_session_ctx = p_enc_ctx->p_session_ctx;
  p_encoder_params = p_enc_ctx->p_encoder_params;

  p_enc_ctx->p_input_fme = (ni_logan_session_data_io_t *)ni_logan_fifo_generic_write(p_enc_ctx->input_data_fifo);
  if (p_enc_ctx->p_input_fme)
  {
    ni_logan_enc_init_aux_params(p_enc_ctx->p_input_fme);

    // force pic qp demo mode: initial QP (200 frames) -> QP value specified by
    // ForcePicQpDemoMode (100 frames) -> initial QP (remaining frames)
    if (p_encoder_params->force_pic_qp_demo_mode)
    {
      if (p_session_ctx->frame_num >= 300)
      {
        p_enc_ctx->p_input_fme->data.frame.force_pic_qp = p_encoder_params->enc_input_params.rc.intra_qp;
      }
      else if (p_session_ctx->frame_num >= 200)
      {
        p_enc_ctx->p_input_fme->data.frame.force_pic_qp = p_encoder_params->force_pic_qp_demo_mode;
      }
    }
  }
  else
  {  
    ni_log(NI_LOG_ERROR, "%s(): Error fifo is full or opinter is null:%p\n",
           __FUNCTION__, p_enc_ctx->input_data_fifo);
    ret = NI_LOGAN_RETCODE_FAILURE;
  }
  return ret;
}

/*!*****************************************************************************
 *  \brief  Fill reconfig vfr
 *
 *  \param[in]  p_ctx  Pointer to ni_logan_enc_context_t
 *  \param[in/out]  ni_frame aux_data may be updated
 *  \param[in]  pts frame pts
 *
 *  \return
 ******************************************************************************/
void ni_logan_encode_reconfig_vfr(ni_logan_enc_context_t * p_enc_ctx,
                                  ni_logan_frame_t *ni_frame,
                                  int64_t pts)
{
  uint32_t timebase = p_enc_ctx->timebase_num / p_enc_ctx->timebase_den;
  ni_logan_enc_reconfig_vfr(p_enc_ctx->p_session_ctx, p_enc_ctx->p_encoder_params, ni_frame, pts, timebase);
  return;
}

/*!*****************************************************************************
 *  \brief  Copy frame data to NI frame
 *
 *  \param[in/out] p_enc_ctx Pointer to ni_logan_enc_context_t
 *  \param[in] p_dec_frame   Pointer to ni_logan_frame_t,
 *                           contains  aux data from NI decoder
 *  \param[in] p_data        Pointer to input frame data
 *  \param[in] linesize      Pointer to linesize of input frame
 *
 *  \return On success
 *                        NI_LOGAN_RETCODE_SUCCESS
 *          On failure
 *                        NI_LOGAN_RETCODE_FAILURE,
 *                        NI_LOGAN_RETCODE_ERROR_MEM_ALOC, etc
 ******************************************************************************/
int ni_logan_encode_copy_frame_data(ni_logan_enc_context_t *p_enc_ctx,
                                    ni_logan_frame_t *p_dec_frame,
                                    uint8_t *p_data[NI_LOGAN_MAX_NUM_DATA_POINTERS],
                                    int linesize[NI_LOGAN_MAX_NUM_DATA_POINTERS])
{
  int ret = NI_LOGAN_RETCODE_SUCCESS;
  ni_logan_session_context_t *p_session_ctx = NULL;
  ni_logan_encoder_params_t *p_encoder_params = NULL;
  ni_logan_frame_t *p_ni_frame = NULL;
  int send_sei_with_idr = 0;

  if (!p_enc_ctx || !p_enc_ctx->p_session_ctx || !p_enc_ctx->p_encoder_params ||
      !p_enc_ctx->p_input_fme)
  {
    ni_log(NI_LOG_ERROR, "%s(): line %d, Invalid Parameter, %p, %p, %p, %p\n",
           __FUNCTION__, __LINE__, p_enc_ctx, p_enc_ctx->p_session_ctx,
           p_enc_ctx->p_encoder_params, p_enc_ctx->p_input_fme);
    return NI_LOGAN_RETCODE_INVALID_PARAM;
  }
  ni_log(NI_LOG_TRACE, "%s() enter\n", __FUNCTION__);
  p_session_ctx = p_enc_ctx->p_session_ctx;
  p_encoder_params = p_enc_ctx->p_encoder_params;
  p_ni_frame = p_enc_ctx->p_input_fme;
  p_ni_frame->video_width = NI_LOGAN_ODD2EVEN(p_ni_frame->video_orig_width);
  p_ni_frame->video_height = NI_LOGAN_ODD2EVEN(p_ni_frame->video_orig_height);

  // data buffer for various SEI: HDR mastering display color volume, HDR
  // content light level, close caption, User data unregistered, HDR10+ etc.
  uint8_t mdcv_data[NI_LOGAN_MAX_SEI_DATA];
  uint8_t cll_data[NI_LOGAN_MAX_SEI_DATA];
  uint8_t cc_data[NI_LOGAN_MAX_SEI_DATA];
  uint8_t udu_data[NI_LOGAN_MAX_SEI_DATA];
  uint8_t hdrp_data[NI_LOGAN_MAX_SEI_DATA];

  int dst_stride[NI_LOGAN_MAX_NUM_DATA_POINTERS] = {0};
  int dst_height_aligned[NI_LOGAN_MAX_NUM_DATA_POINTERS] = {0};
  int src_height[NI_LOGAN_MAX_NUM_DATA_POINTERS] = {0};

  // whether should send SEI with this frame
  send_sei_with_idr = ni_logan_should_send_sei_with_frame(p_session_ctx,
                                                          p_ni_frame->ni_logan_pict_type,
                                                          p_encoder_params);

  // prep for auxiliary data (various SEI, ROI) in encode frame, based on the
  // data returned in decoded frame
  ni_logan_enc_prep_aux_data(p_session_ctx, p_ni_frame, p_dec_frame,
                             p_enc_ctx->codec_format, send_sei_with_idr,
                             mdcv_data, cll_data, cc_data, udu_data, hdrp_data);

  if (p_ni_frame->sei_total_len > NI_LOGAN_ENC_MAX_SEI_BUF_SIZE)
  {
    ni_log(NI_LOG_ERROR, "%s: sei total length %u exceeds maximum sei "
           "size %u.\n", __FUNCTION__, p_ni_frame->sei_total_len,
           NI_LOGAN_ENC_MAX_SEI_BUF_SIZE);
    ret = NI_LOGAN_RETCODE_FAILURE;
    return ret;
  }

  p_ni_frame->extra_data_len += p_ni_frame->sei_total_len;
  // FW layout requirement: leave space for reconfig data if SEI and/or ROI
  // is present
  if ((p_ni_frame->sei_total_len || p_ni_frame->roi_len)
      && !p_ni_frame->reconf_len)
  {
    p_ni_frame->extra_data_len += sizeof(ni_logan_encoder_change_params_t);
  }
  
  src_height[0] = p_ni_frame->video_orig_height;
  src_height[1] = src_height[2] = src_height[0] / 2;

  ni_logan_get_hw_yuv420p_dim(p_ni_frame->video_width,
                              p_ni_frame->video_height,
                              p_session_ctx->bit_depth_factor,
                              p_enc_ctx->codec_format == NI_LOGAN_CODEC_FORMAT_H264,
                              dst_stride, dst_height_aligned);
  
  // alignment(16) extra padding for H.264 encoding
  ni_logan_encoder_frame_buffer_alloc(p_ni_frame,
                                      p_ni_frame->video_width,
                                      p_ni_frame->video_height,
                                      dst_stride,
                                      (p_enc_ctx->codec_format == NI_LOGAN_CODEC_FORMAT_H264),
                                      p_ni_frame->extra_data_len,
                                      p_session_ctx->bit_depth_factor);
  if (!p_ni_frame->p_data[0])
  {
    ni_log(NI_LOG_ERROR, "%s: alloc frame buffer failed\n", __FUNCTION__);
    ret = NI_LOGAN_RETCODE_ERROR_MEM_ALOC;
    return ret;
  }

  ni_log(NI_LOG_TRACE, "%s: api_fme.data_len[0]=%d, "
         "buffered_fme.linesize=%d/%d/%d, dst alloc linesize = %d/%d/%d, "
         "src height = %d/%d%d, dst height aligned = %d/%d/%d, "
         "ctx->api_fme.force_key_frame=%d, extra_data_len=%d sei_size=%u "
         "(hdr_content_light_level %u hdr_mastering_display_color_vol %u "
         "hdr10+ %u cc %u udu %u prefC %u) "
         "reconf_size=%u roi_size=%u force_pic_qp=%u "
         "use_cur_src_as_long_term_pic %u use_long_term_ref %u\n",
         __FUNCTION__, p_ni_frame->data_len[0], linesize[0], linesize[1],
         linesize[2], dst_stride[0], dst_stride[1], dst_stride[2],
         src_height[0], src_height[1], src_height[2],
         dst_height_aligned[0], dst_height_aligned[1], dst_height_aligned[2],
         p_ni_frame->force_key_frame,
         p_ni_frame->extra_data_len,
         p_ni_frame->sei_total_len,
         p_ni_frame->sei_hdr_content_light_level_info_len,
         p_ni_frame->sei_hdr_mastering_display_color_vol_len,
         p_ni_frame->sei_hdr_plus_len,
         p_ni_frame->sei_cc_len,
         p_ni_frame->sei_user_data_unreg_len,
         p_ni_frame->preferred_characteristics_data_len,
         p_ni_frame->reconf_len,
         p_ni_frame->roi_len,
         p_ni_frame->force_pic_qp,
         p_ni_frame->use_cur_src_as_long_term_pic,
         p_ni_frame->use_long_term_ref);
  
  // YUV part of the encoder input data layout
  ni_logan_copy_hw_yuv420p(p_ni_frame->p_data, p_data,
                           p_ni_frame->video_orig_width,
                           p_ni_frame->video_orig_height,
                           p_session_ctx->bit_depth_factor,
                           dst_stride, dst_height_aligned,
                           linesize, src_height);

  ni_log(NI_LOG_TRACE, "%s: After memcpy p_data 0:0x%p 1:0x%p 2:0x%p "
         "len:0:%d 1:%d 2:%d\n",
         p_ni_frame->p_data,
         p_ni_frame->p_data,
         p_ni_frame->p_data,
         p_ni_frame->data_len[0],
         p_ni_frame->data_len[1],
         p_ni_frame->data_len[2]);

  // auxiliary data part of the encoder input data layout
  ni_logan_enc_copy_aux_data(p_session_ctx, p_ni_frame, p_dec_frame,
                        p_session_ctx->codec_format, mdcv_data, cll_data,
                        cc_data, udu_data, hdrp_data);
  return ret;
}

/*!*****************************************************************************
 *  \brief  Send encode data to NI device
 *
 *  \param[in] p_enc_ctx  Pointer to ni_logan_enc_context_t
 *
 *  \return On success
 *                        NI_LOGAN_RETCODE_SUCCESS
 *          On failure
 *                        NI_LOGAN_RETCODE_FAILURE,
 *                        NI_LOGAN_RETCODE_INVALID_PARAM, etc.
 ******************************************************************************/
int ni_logan_encode_send(ni_logan_enc_context_t *p_enc_ctx)
{
  int ret = NI_LOGAN_RETCODE_SUCCESS;
  int sent;
  ni_logan_session_context_t *p_session_ctx = NULL;
  ni_logan_encoder_params_t *p_encoder_params = NULL;
  ni_logan_session_data_io_t *p_input_fme = NULL;

  if (!p_enc_ctx || !p_enc_ctx->p_session_ctx || !p_enc_ctx->p_encoder_params)
  {
    ni_log(NI_LOG_ERROR, "%s(): line %d, Invalid Parameter, %p, %p, %p\n",
           __FUNCTION__, __LINE__, p_enc_ctx, p_enc_ctx->p_session_ctx,
           p_enc_ctx->p_encoder_params);
    return NI_LOGAN_RETCODE_INVALID_PARAM;
  }
  ni_log(NI_LOG_TRACE, "%s() enter\n", __FUNCTION__);

  p_session_ctx = p_enc_ctx->p_session_ctx;
  p_encoder_params = p_enc_ctx->p_encoder_params;

  if (p_enc_ctx->encoder_flushing)
  {
    if (ni_logan_fifo_is_empty(p_enc_ctx->input_data_fifo))
    {
      ni_log(NI_LOG_DEBUG, "XCoder EOF: null frame && fifo empty\n");
      return ret;
    }
  }

  if (LOGAN_SESSION_RUN_STATE_SEQ_CHANGE_DRAINING == p_session_ctx->session_run_state)
  {
    ni_log(NI_LOG_DEBUG, "%s(): sequence change draining state\n", __FUNCTION__);
    return ret;
  }

  p_input_fme = ni_logan_fifo_generic_read(p_enc_ctx->input_data_fifo);

  if (!p_input_fme && p_enc_ctx->eos_fme_received)
  {
    p_enc_ctx->encoder_flushing = 1;
    ni_log(NI_LOG_DEBUG, "null frame, eos_fme_received = 1\n");
    if (p_enc_ctx->started)
    {
      ni_logan_device_session_flush(p_session_ctx, NI_LOGAN_DEVICE_TYPE_ENCODER);
    }
    return ret;
  }
  else
  {
    p_enc_ctx->p_input_fme = p_input_fme;

    // check whether sequence changed
    if (p_input_fme->data.frame.video_orig_width != p_enc_ctx->width ||
        p_input_fme->data.frame.video_orig_height != p_enc_ctx->height)
    {
      ni_logan_device_session_flush(p_session_ctx, NI_LOGAN_DEVICE_TYPE_ENCODER);
      ret = ni_logan_fifo_return_read(p_enc_ctx->input_data_fifo);
      if (ret < 0)
      {
        ni_log(NI_LOG_INFO, "%s(): Drop 1 frame because of buffer full, "
               "when sequence changed\n", __FUNCTION__);
      }

      p_session_ctx->session_run_state = LOGAN_SESSION_RUN_STATE_SEQ_CHANGE_DRAINING;
      ni_log(NI_LOG_TRACE, "%s(): frame_num=%d, encoder sequence changed, "
             "%dx%d to %dx%d\n", __FUNCTION__, p_session_ctx->frame_num,
             p_enc_ctx->width, p_enc_ctx->height,
             p_input_fme->data.frame.video_orig_width,
             p_input_fme->data.frame.video_orig_height);
      return ret;
    }

    ni_log(NI_LOG_TRACE, "%s(): ni_frame num=%d, pts=%d, dts=%d, WxH=%dx%d, "
           "orig WxH=%dx%d, crop=%d,%d,%d,%d, pict_type=%d, key_frame=%d, "
           "buffer_size=%d, data_len=%d,%d,%d,%d\n", __FUNCTION__,
           p_session_ctx->frame_num,
           p_input_fme->data.frame.pts,
           p_input_fme->data.frame.dts,
           p_input_fme->data.frame.video_width,
           p_input_fme->data.frame.video_height,
           p_input_fme->data.frame.video_orig_width,
           p_input_fme->data.frame.video_orig_height,
           p_input_fme->data.frame.crop_top,
           p_input_fme->data.frame.crop_bottom,
           p_input_fme->data.frame.crop_left,
           p_input_fme->data.frame.crop_right,
           p_input_fme->data.frame.ni_logan_pict_type,
           p_input_fme->data.frame.force_key_frame,
           p_input_fme->data.frame.buffer_size,
           p_input_fme->data.frame.data_len[0],
           p_input_fme->data.frame.data_len[1],
           p_input_fme->data.frame.data_len[2],
           p_input_fme->data.frame.data_len[3]);
    ni_log(NI_LOG_TRACE, "%s(): ni_frame sei_total_len=%d, sei_cc=%d-%d, "
           "sei_hdr_mastering_display_color_vol=%d-%d, sei_hdr_plus=%d-%d, "
           "sei_hdr_content_light_level_info=%d-%d, sei_user_data_unreg=%d-%d, "
           "sei_alt_transfer_characteristics=%d-%d, vui=%d-%d, roi_len=%d, "
           "reconf_len=%d, extra_data_len=%d\n", __FUNCTION__,
           p_input_fme->data.frame.sei_total_len,
           p_input_fme->data.frame.sei_cc_offset,
           p_input_fme->data.frame.sei_cc_len,
           p_input_fme->data.frame.sei_hdr_mastering_display_color_vol_offset,
           p_input_fme->data.frame.sei_hdr_mastering_display_color_vol_len,
           p_input_fme->data.frame.sei_hdr_plus_offset,
           p_input_fme->data.frame.sei_hdr_plus_len,
           p_input_fme->data.frame.sei_hdr_content_light_level_info_offset,
           p_input_fme->data.frame.sei_hdr_content_light_level_info_len,
           p_input_fme->data.frame.sei_user_data_unreg_offset,
           p_input_fme->data.frame.sei_user_data_unreg_len,
           p_input_fme->data.frame.sei_alt_transfer_characteristics_offset,
           p_input_fme->data.frame.sei_alt_transfer_characteristics_len,
           p_input_fme->data.frame.vui_offset,
           p_input_fme->data.frame.vui_len,
           p_input_fme->data.frame.roi_len,
           p_input_fme->data.frame.reconf_len,
           p_input_fme->data.frame.extra_data_len);

    if (p_enc_ctx->started == 0)
    {
      ret = ni_logan_encode_open(p_enc_ctx);
      if (ret < 0)
      {
        ni_log(NI_LOG_ERROR, "ni_logan_encode_open failed:%d\n", ret);
        return ret;
      }
      p_enc_ctx->started = 1;
      p_input_fme->data.frame.start_of_stream = 1;
    }
    else
    {
      p_input_fme->data.frame.start_of_stream = 0;
    }

    sent = ni_logan_device_session_write(p_session_ctx, p_input_fme, NI_LOGAN_DEVICE_TYPE_ENCODER);
    ni_log(NI_LOG_TRACE, "%s: pts %lld dts %lld size %d sent to xcoder\n",
           __FUNCTION__, p_input_fme->data.frame.pts, p_input_fme->data.frame.dts, sent);
    // return EIO at error
    if (NI_LOGAN_RETCODE_ERROR_VPU_RECOVERY == sent)
    {
      ni_log(NI_LOG_TRACE, "%s(): vpu recovery1, push back frame %d\n",
             __FUNCTION__, p_session_ctx->frame_num);
      p_session_ctx->session_run_state = LOGAN_SESSION_RUN_STATE_RESETTING;
      ret = ni_logan_fifo_return_read(p_enc_ctx->input_data_fifo);
      if (ret < 0)
      {
        ni_log(NI_LOG_INFO, "%s(): Drop 1 frame because of buffer full, "
               "when vpu recovery happened\n", __FUNCTION__);
      }
      ret = ni_logan_encode_reinit(p_enc_ctx);
      if (ret < 0)
      {
        ni_log(NI_LOG_ERROR, "%s(): VPU recovery failed:%d, returning "
               "EIO\n", __FUNCTION__, sent);
        ret = NI_LOGAN_RETCODE_FAILURE;
      }
      return ret;
    }
    else if (sent < 0)
    {
      ni_log(NI_LOG_ERROR, "%s(): failure sent (%d), returning EIO\n",
             __FUNCTION__, sent);
      return ret;
    }
    else if (sent == 0)
    {
      // when buffer_full, drop the frame and return EAGAIN if in strict timeout
      // mode, otherwise buffer the frame and it is to be sent out using encode2
      // API: queue the frame only if not done so yet, i.e. queue is empty
      // *and* it's a valid frame. ToWatch: what are other rc cases ?
      if (p_session_ctx->status == NI_LOGAN_RETCODE_NVME_SC_WRITE_BUFFER_FULL)
      {
        if (p_encoder_params->strict_timeout_mode)
        {
          ni_log(NI_LOG_ERROR, "%s(): Error Strict timeout period exceeded"
                 " returning EAGAIN\n", __FUNCTION__);
          ret = NI_LOGAN_RETCODE_FAILURE;
        }
        else
        {
          ni_log(NI_LOG_TRACE, "%s(): Write buffer full, push back frame %d\n",
                 __FUNCTION__, p_session_ctx->frame_num);
          ret = ni_logan_fifo_return_read(p_enc_ctx->input_data_fifo);
          if (ret < 0)
          {
            ni_log(NI_LOG_INFO, "%s(): Drop 1 frame because of fifo full, "
                   "when wirte buffer full\n", __FUNCTION__);
          }
        }
        return ret;
      }
    }
    else
    {
      p_enc_ctx->sentFrame = 1;
      //pushing input pts in circular FIFO
      p_session_ctx->enc_pts_list[p_session_ctx->enc_pts_w_idx % NI_LOGAN_FIFO_SZ] = p_input_fme->data.frame.pts;
      p_session_ctx->enc_pts_w_idx++;
      ret = NI_LOGAN_RETCODE_SUCCESS;
    }
  }

  return ret;
}

/*!*****************************************************************************
 *  \brief  Copy NI packet to packet data buffer
 *
 *  \param[in] p_enc_ctx    Pointer to ni_logan_enc_context_t
 *  \param[out] p_data      Pointer to output packet data buffer,
 *  \param[in] first_packet flag of first packet
 *  \param[in] sps_pps_attach  flag of attached SPS/PPS
 *
 *  \return On success
 *                        NI_LOGAN_RETCODE_SUCCESS
 *          On failure
 *                        NI_LOGAN_RETCODE_FAILURE,
 *                        NI_LOGAN_RETCODE_PARAM_ERROR_WIDTH_TOO_BIG, etc
 ******************************************************************************/
int ni_logan_encode_copy_packet_data(ni_logan_enc_context_t *p_enc_ctx,
                                     uint8_t *p_data,
                                     int first_packet,
                                     int sps_pps_attach)
{
  int ret = 0;
  ni_logan_session_context_t *p_session_ctx = NULL;
  ni_logan_encoder_params_t *p_encoder_params = NULL;
  ni_logan_packet_t *p_output_pkt = NULL;

  if (!p_enc_ctx || !p_enc_ctx->p_session_ctx || !p_enc_ctx->p_encoder_params)
  {
    ni_log(NI_LOG_ERROR, "%s(): line %d, Invalid Parameter, %p, %p, %p\n",
           __FUNCTION__, __LINE__, p_enc_ctx, p_enc_ctx->p_session_ctx,
           p_enc_ctx->p_encoder_params);
    return NI_LOGAN_RETCODE_INVALID_PARAM;
  }
  ni_log(NI_LOG_TRACE, "%s() enter\n", __FUNCTION__);

  p_session_ctx = p_enc_ctx->p_session_ctx;
  p_encoder_params = p_enc_ctx->p_encoder_params;
  p_output_pkt = &p_enc_ctx->output_pkt;

  int meta_size = NI_LOGAN_FW_ENC_BITSTREAM_META_DATA_SIZE;
  int nalu_type = 0;
  const uint8_t *p_start_code;
  uint32_t stc = -1;
  uint32_t copy_len = 0;
  uint8_t *p_src = (uint8_t*)p_output_pkt->p_data + meta_size;
  uint8_t *p_end = p_src + (p_output_pkt->data_len - meta_size);
  int64_t local_pts = p_output_pkt->pts;
  int custom_sei_cnt = 0;
  int sei_idx = 0;
  ni_logan_all_custom_sei_t *ni_logan_all_custom_sei;
  ni_logan_custom_sei_t *ni_logan_custom_sei;
  if (p_session_ctx->pkt_custom_sei[local_pts % NI_LOGAN_FIFO_SZ])
  {
    ni_logan_all_custom_sei = p_session_ctx->pkt_custom_sei[local_pts % NI_LOGAN_FIFO_SZ];
    custom_sei_cnt = ni_logan_all_custom_sei->custom_sei_cnt;
  }

  if (custom_sei_cnt)
  {
    // if HRD or custom sei enabled, search for pic_timing or custom SEI insertion point by
    // skipping non-VCL until video data is found.
    p_start_code = p_src;
    if(NI_LOGAN_CODEC_FORMAT_H265 == p_enc_ctx->codec_format)
    {
      do
      {
        stc = -1;
        p_start_code = ni_logan_find_start_code(p_start_code, p_end, &stc);
        nalu_type = (stc >> 1) & 0x3F;
      } while (nalu_type > 21/*NAL_CRA_NUT*/);
  
      // calc. length to copy
      copy_len = p_start_code - 5 - p_src;
    }
    else if(NI_LOGAN_CODEC_FORMAT_H264 == p_enc_ctx->codec_format)
    {
      do
      {
        stc = -1;
        p_start_code = ni_logan_find_start_code(p_start_code, p_end, &stc);
        nalu_type = stc & 0x1F;
      } while (nalu_type > 5/*NAL_IDR_SLICE*/);
  
      // calc. length to copy
      copy_len = p_start_code - 5 - p_src;
    }
    else
    {
      ni_log(NI_LOG_ERROR, "%s: codec %d not supported for SEI !\n",
             __FUNCTION__, p_enc_ctx->codec_format);
    }
  }

  uint8_t *p_dst = p_data;
  if (sps_pps_attach)
  {
    memcpy(p_dst, p_enc_ctx->p_spsPpsHdr, p_enc_ctx->spsPpsHdrLen);
    p_dst += p_enc_ctx->spsPpsHdrLen;
  }
  if (custom_sei_cnt)
  {
    // copy buf_period
    memcpy(p_dst, p_src, copy_len);
    p_dst += copy_len;

    // copy custom sei before slice
    sei_idx = 0;
    while (sei_idx < custom_sei_cnt)
    {
      ni_logan_custom_sei = &ni_logan_all_custom_sei->ni_custom_sei[sei_idx];
      if (ni_logan_custom_sei->custom_sei_loc == NI_LOGAN_CUSTOM_SEI_LOC_AFTER_VCL)
      {
        break;
      }
      memcpy(p_dst, ni_logan_custom_sei->custom_sei_data, ni_logan_custom_sei->custom_sei_size);
      p_dst += ni_logan_custom_sei->custom_sei_size;
      sei_idx ++;
    }

    // copy the IDR data
    memcpy(p_dst, p_src + copy_len,
           p_output_pkt->data_len - meta_size - copy_len);
    p_dst += (p_output_pkt->data_len - meta_size - copy_len);

    // copy custom sei after slice
    while (sei_idx < custom_sei_cnt)
    {
      ni_logan_custom_sei = &ni_logan_all_custom_sei->ni_custom_sei[sei_idx];
      memcpy(p_dst, ni_logan_custom_sei->custom_sei_data, ni_logan_custom_sei->custom_sei_size);
      p_dst += ni_logan_custom_sei->custom_sei_size;
      sei_idx ++;
    }
  }
  else
  {
    memcpy(p_dst, (uint8_t*)p_output_pkt->p_data + meta_size,
           p_output_pkt->data_len - meta_size);
  }

  // free buffer
  if (custom_sei_cnt)
  {
    free(p_session_ctx->pkt_custom_sei[local_pts % NI_LOGAN_FIFO_SZ]);
    p_session_ctx->pkt_custom_sei[local_pts % NI_LOGAN_FIFO_SZ] = NULL;
  }

  int pts, dts;
  pts = p_output_pkt->pts;
  /* to ensure pts>dts for all frames, we assign a guess pts for the first 'dts_offset' frames and then the pts from input stream
   * is extracted from input pts FIFO.
   * if GOP = IBBBP and PTSs = 0 1 2 3 4 5 .. then out DTSs = -3 -2 -1 0 1 ... and -3 -2 -1 are the guessed values
   * if GOP = IBPBP and PTSs = 0 1 2 3 4 5 .. then out DTSs = -1 0 1 2 3 ... and -1 is the guessed value
   * the number of guessed values is equal to dts_offset
   */
  if (p_enc_ctx->total_frames_received < p_enc_ctx->dts_offset)
  {
    // guess dts
    dts = p_enc_ctx->first_frame_pts + (p_enc_ctx->total_frames_received - p_enc_ctx->dts_offset) * p_enc_ctx->ticks_per_frame;
  }
  else
  {
    // get dts from pts FIFO
    dts = p_session_ctx->enc_pts_list[p_session_ctx->enc_pts_r_idx % NI_LOGAN_FIFO_SZ];
    p_session_ctx->enc_pts_r_idx++;
  }

  if (p_enc_ctx->total_frames_received >= 1)
  {
    if (dts < p_enc_ctx->latest_dts)
    {
      ni_log(NI_LOG_INFO, "dts: %ld < latest_dts: %ld.\n",
             dts, p_enc_ctx->latest_dts);
    }
  }

  if (dts > pts)
  {
    ni_log(NI_LOG_INFO, "dts: %ld, pts: %ld. Forcing dts=pts\n",
           dts, pts);
    dts = pts;
  }
  p_enc_ctx->total_frames_received++;
  p_enc_ctx->latest_dts = dts;

  p_output_pkt->pts = pts;
  p_output_pkt->dts = dts;
  ni_log(NI_LOG_DEBUG, "%s pkt %" PRId64 " pts %" PRId64 " "
         "dts %" PRId64 " \n", __FUNCTION__,
         p_session_ctx->pkt_num - 1, pts, dts);

  p_enc_ctx->encoder_eof = p_output_pkt->end_of_stream;

  if (p_enc_ctx->encoder_eof &&
      LOGAN_SESSION_RUN_STATE_SEQ_CHANGE_DRAINING == p_session_ctx->session_run_state)
  {
    // after sequence change completes, reset codec state
    ni_log(NI_LOG_TRACE, "%s(): sequence change2 completed, return 0 "
           "and will reopen codec !\n", __FUNCTION__);
    ret = ni_logan_encode_reinit(p_enc_ctx);
  }
  else if(!ni_logan_fifo_is_empty(p_enc_ctx->input_data_fifo) &&
          (LOGAN_SESSION_RUN_STATE_NORMAL == p_session_ctx->session_run_state) &&
          (NI_LOGAN_RETCODE_NVME_SC_WRITE_BUFFER_FULL != p_session_ctx->status))
  {
    p_session_ctx->session_run_state = LOGAN_SESSION_RUN_STATE_QUEUED_FRAME_DRAINING;
    ret = ni_logan_encode_send(p_enc_ctx);
    // if session_run_state is changed in xcoder_send_frame, keep it
    if (LOGAN_SESSION_RUN_STATE_QUEUED_FRAME_DRAINING == p_session_ctx->session_run_state)
    {
      p_session_ctx->session_run_state = LOGAN_SESSION_RUN_STATE_NORMAL;
    }
    if (ret < 0)
    {
      ni_log(NI_LOG_ERROR, "ni_logan_encode_send flush send failed ret = %d\n",
                ret);
      return ret;
    }
  }
  return ret;
}

/*!*****************************************************************************
 *  \brief  Receive encoded data from NI device
 *
 *  \param[in] p_enc_ctx  Pointer to ni_logan_enc_context_t
 *
 *  \return 0 when no packets received
 *          > 0 when new packet received, means the size of packet
 *          < 0 when failed, NI_LOGAN_RETCODE_FAILURE,
 *                           NI_LOGAN_RETCODE_INVALID_PARAM, etc.
 ******************************************************************************/
int ni_logan_encode_receive(ni_logan_enc_context_t *p_enc_ctx)
{
  int ret = 0;
  int recv;
  int i;
  ni_logan_session_context_t *p_session_ctx = NULL;
  ni_logan_encoder_params_t *p_encoder_params = NULL;
  ni_logan_packet_t *p_output_pkt = NULL;
  
  if (!p_enc_ctx || !p_enc_ctx->p_session_ctx || !p_enc_ctx->p_encoder_params)
  {
    ni_log(NI_LOG_ERROR, "%s(): line %d, Invalid Parameter, %p, %p, %p\n",
           __FUNCTION__, __LINE__, p_enc_ctx, p_enc_ctx->p_session_ctx,
           p_enc_ctx->p_encoder_params);
    return NI_LOGAN_RETCODE_INVALID_PARAM;
  }
  ni_log(NI_LOG_TRACE, "%s() enter\n", __FUNCTION__);

  p_session_ctx = p_enc_ctx->p_session_ctx;
  p_encoder_params = p_enc_ctx->p_encoder_params;
  p_output_pkt = &p_enc_ctx->output_pkt;

  ni_logan_packet_buffer_alloc(p_output_pkt, NI_LOGAN_MAX_TX_SZ);

  for (; ;)
  {
    recv = ni_logan_device_session_read(p_session_ctx, p_output_pkt, NI_LOGAN_DEVICE_TYPE_ENCODER);

    ni_log(NI_LOG_TRACE, "%s(): pkt_num=%d, end_of_stream=%d, data_len=%d, "
           "recv=%d, encoder_flushing=%d, encoder_eof=%d\n",
           __FUNCTION__, p_session_ctx->pkt_num, p_output_pkt->end_of_stream,
           p_output_pkt->data_len, recv, p_enc_ctx->encoder_flushing,
           p_enc_ctx->encoder_eof);

    if (recv <= 0)
    {
      p_enc_ctx->encoder_eof = p_output_pkt->end_of_stream;
      /* not ready ?? */
      if (p_enc_ctx->encoder_eof || p_output_pkt->end_of_stream)
      {
        if (LOGAN_SESSION_RUN_STATE_SEQ_CHANGE_DRAINING == p_session_ctx->session_run_state)
        {
          // after sequence change completes, reset codec state
          ni_log(NI_LOG_TRACE, "%s(): sequence change1 completed, return "
                 "AVERROR(EAGAIN) and will reopen codec!\n", __FUNCTION__);
          ret = ni_logan_encode_reinit(p_enc_ctx);
          if (ret >= 0)
          {
            // when reopen encoder success, try to recive packet
            ni_logan_packet_buffer_alloc(p_output_pkt, NI_LOGAN_MAX_TX_SZ);
            continue;
            // ret = 0;
          }
          break;
        }

        ret = 0;
        ni_log(NI_LOG_TRACE, "%s: got encoder_eof return AVERROR_EOF\n",
               __FUNCTION__);
        break;
      }
      else
      {
        if (NI_LOGAN_RETCODE_ERROR_VPU_RECOVERY == recv)
        {
          ni_log(NI_LOG_DEBUG, "%s(): vpu recovery2\n", __FUNCTION__);
          p_session_ctx->session_run_state = LOGAN_SESSION_RUN_STATE_RESETTING;
          ret = ni_logan_encode_reinit(p_enc_ctx);
          if (ret < 0)
          {
            ni_log(NI_LOG_ERROR, "%s(): VPU recovery failed:%d, "
                   "returning EIO\n", __FUNCTION__, ret);
            ret = NI_LOGAN_RETCODE_FAILURE;
          }
          return ret;
        }
        else if (recv < 0)
        {
          if ((NI_LOGAN_RETCODE_ERROR_INVALID_SESSION == recv) && !p_enc_ctx->started)
          {
            // session may be in recovery state, return EAGAIN
            ni_log(NI_LOG_INFO, "%s: VPU might be reset, "
                   "invalid session id\n", __FUNCTION__);
            ret = 0;
          }
          else
          {
            ni_log(NI_LOG_ERROR, "%s: Persistent failure, "
                   "returning EIO,ret=%d\n", __FUNCTION__, recv);
            ret = NI_LOGAN_RETCODE_FAILURE;
          }
          p_enc_ctx->gotPacket = 0;
          p_enc_ctx->sentFrame = 0;
          break;
        }
        else if (p_encoder_params->low_delay_mode == 1 && p_enc_ctx->sentFrame && !p_enc_ctx->gotPacket)
        {
          ni_log(NI_LOG_TRACE, "%s: low delay mode, keep reading until "
                 "pkt arrives\n", __FUNCTION__);
          continue;
        }
        else if (p_encoder_params->low_delay_mode == 2 &&
                 p_session_ctx->frame_num - p_session_ctx->pkt_num >
                  (p_encoder_params->enc_input_params.gop_preset_index == 0 ?
                   p_encoder_params->enc_input_params.custom_gop_params.custom_gop_size:
                   g_map_preset_to_gopsize[p_encoder_params->enc_input_params.gop_preset_index]))
        {
          ni_log(NI_LOG_TRACE, "%s: low delay mode 2, keep reading send frame %d receive pkt %d gop %d\n",
                 __FUNCTION__, p_session_ctx->frame_num, p_session_ctx->pkt_num,
                 p_encoder_params->enc_input_params.gop_preset_index);
          continue;
        }
        p_enc_ctx->gotPacket = 0;
        p_enc_ctx->sentFrame = 0;

        if (!ni_logan_fifo_is_empty(p_enc_ctx->input_data_fifo) &&
            (LOGAN_SESSION_RUN_STATE_NORMAL == p_session_ctx->session_run_state) &&
            (NI_LOGAN_RETCODE_NVME_SC_WRITE_BUFFER_FULL != p_session_ctx->status))
        {
            // flush the fifo
            p_session_ctx->session_run_state = LOGAN_SESSION_RUN_STATE_QUEUED_FRAME_DRAINING;
            ret = ni_logan_encode_send(p_enc_ctx);
            // if session_run_state is changed in xcoder_send_frame, keep it
            if (LOGAN_SESSION_RUN_STATE_QUEUED_FRAME_DRAINING == p_session_ctx->session_run_state)
            {
              p_session_ctx->session_run_state = LOGAN_SESSION_RUN_STATE_NORMAL;
            }

            if (ret < 0)
            {
              ni_log(NI_LOG_ERROR, "ni_logan_encode_send flush send failed ret = %d\n",
                       ret);
              return ret;
            }
            continue;
        }
        ret = 0;
        if (!p_enc_ctx->encoder_flushing && !p_enc_ctx->eos_fme_received)
        {
          ni_log(NI_LOG_TRACE, "%s: NOT encoder_flushing, return "
                 "AVERROR(EAGAIN)\n", __FUNCTION__);
          break;
        }
      }
    }
    else
    {
      /* got encoded data back */
      int meta_size = NI_LOGAN_FW_ENC_BITSTREAM_META_DATA_SIZE;
      if (! p_enc_ctx->spsPpsArrived)
      {
        ret = 0;
        p_enc_ctx->spsPpsArrived = 1;
        p_enc_ctx->spsPpsHdrLen = recv - meta_size;
        p_enc_ctx->p_spsPpsHdr = malloc(p_enc_ctx->spsPpsHdrLen);
        if (! p_enc_ctx->p_spsPpsHdr)
        {
          ret = NI_LOGAN_RETCODE_FAILURE;
          break;
        }

        memcpy(p_enc_ctx->p_spsPpsHdr, (uint8_t*)p_output_pkt->p_data + meta_size,
               p_output_pkt->data_len - meta_size);
 
        // start pkt_num counter from 1 to get the real first frame
        p_session_ctx->pkt_num = 1;
        // for low-latency mode, keep reading until the first frame is back
        if (p_encoder_params->low_delay_mode == 1)
        {
          ni_log(NI_LOG_TRACE, "%s: low delay mode, keep reading until "
                 "1ast pkt arrives\n", __FUNCTION__);
          continue;
        }
        // when recive sps, should try to recive the first packet
        continue;
      }
      p_enc_ctx->gotPacket = 1;
      p_enc_ctx->sentFrame = 0;

      int nalu_type = 0;
      const uint8_t *p_start_code;
      uint32_t stc = -1;
      uint32_t copy_len = 0;
      uint8_t *p_src = (uint8_t*)p_output_pkt->p_data + meta_size;
      uint8_t *p_end = p_src + (p_output_pkt->data_len - meta_size);
      int64_t local_pts = p_output_pkt->pts;
      int custom_sei_cnt = 0;
      int total_custom_sei_len = 0;
      int sei_idx = 0;
      ni_logan_all_custom_sei_t *ni_logan_all_custom_sei;
      if (p_session_ctx->pkt_custom_sei[local_pts % NI_LOGAN_FIFO_SZ])
      {
        ni_logan_all_custom_sei = p_session_ctx->pkt_custom_sei[local_pts % NI_LOGAN_FIFO_SZ];
        custom_sei_cnt = ni_logan_all_custom_sei->custom_sei_cnt;
        for (sei_idx = 0; sei_idx < custom_sei_cnt; sei_idx ++)
        {
          total_custom_sei_len += ni_logan_all_custom_sei->ni_custom_sei[sei_idx].custom_sei_size;
        }
      }

      if (!p_enc_ctx->firstPktArrived)
      {
        p_enc_ctx->spsPpsAttach = 0;
        // if enable forced repeat header, add a SPS/PPS header in the first packat
        if ((p_encoder_params->enc_input_params.forced_header_enable == NI_LOGAN_ENC_REPEAT_HEADERS_ALL_KEY_FRAMES) ||
            (p_encoder_params->enc_input_params.forced_header_enable == NI_LOGAN_ENC_REPEAT_HEADERS_ALL_I_FRAMES))
        {
          p_enc_ctx->spsPpsAttach = 1;
        }
      }
      else
      {      
        p_enc_ctx->spsPpsAttach = 0;
        // insert header when intraRefresh is enabled and forced header mode is 1 (all key frames)
        // for every intraRefreshMinPeriod key frames, pkt counting starts from 1, e.g. for
        // cycle of 100, the header is forced on frame 102, 202, ...;
        // note that api_ctx.pkt_num returned is the actual index + 1
        if (p_enc_ctx->p_spsPpsHdr && p_enc_ctx->spsPpsHdrLen &&
            (p_encoder_params->enc_input_params.forced_header_enable == NI_LOGAN_ENC_REPEAT_HEADERS_ALL_KEY_FRAMES) &&
            (1 == p_encoder_params->enc_input_params.intra_mb_refresh_mode ||
             2 == p_encoder_params->enc_input_params.intra_mb_refresh_mode ||
             3 == p_encoder_params->enc_input_params.intra_mb_refresh_mode))
        {
          if (p_encoder_params->intra_refresh_reset)
          {
            if (p_session_ctx->pkt_num - p_session_ctx->force_frame_pkt_num == p_encoder_params->ui32minIntraRefreshCycle)
            {
              p_enc_ctx->spsPpsAttach = 1;
              p_session_ctx->force_frame_pkt_num = p_session_ctx->pkt_num;
            }
            else
            {
              for (i = 0; i < NI_LOGAN_MAX_FORCE_FRAME_TABLE_SIZE; i++)
              {
                if (p_output_pkt->pts == p_session_ctx->force_frame_pts_table[i])
                {
                  p_enc_ctx->spsPpsAttach = 1;
                  p_session_ctx->force_frame_pkt_num = p_session_ctx->pkt_num;
                  break;
                }
              }
            }
          }
          else if (p_encoder_params->ui32minIntraRefreshCycle > 0 && p_session_ctx->pkt_num > 3 &&
                   0 == ((p_session_ctx->pkt_num - 3) % p_encoder_params->ui32minIntraRefreshCycle))
          {
            p_enc_ctx->spsPpsAttach = 1;
            ni_log(NI_LOG_TRACE, "%s pkt %" PRId64 " force header on "
                   "intraRefreshMinPeriod %u\n", __FUNCTION__,
                   p_session_ctx->pkt_num - 1, p_encoder_params->ui32minIntraRefreshCycle);
          }
        }
      }

      ret = p_output_pkt->data_len - meta_size + total_custom_sei_len;
      break;
    }
  }
  return ret;
}
